import unittest

from dropbox_sign import ApiClient, Configuration, apis, models as m
from test_utils import get_fixture_data, MockPoolManager, get_base_path


class TestTemplateApi(unittest.TestCase):
    def setUp(self):
        self.configuration = Configuration()
        self.api_client = ApiClient(self.configuration)
        self.mock_pool = MockPoolManager(self)
        self.api_client.rest_client.pool_manager = self.mock_pool

        self.api = apis.TemplateApi(self.api_client)

    def test_template_add_user(self):
        template_id = 'f57db65d3f933b5316d398057a36176831451a35'

        request_class = 'TemplateAddUserRequest'
        request_data = get_fixture_data(request_class)['default']

        response_class = 'TemplateGetResponse'
        response_data = get_fixture_data(response_class)['default']

        self.mock_pool.expect_request(
            content_type='application/json',
            data=request_data,
            response=response_data
        )
        expected = m.TemplateGetResponse.init(response_data)
        obj = m.TemplateAddUserRequest.init(request_data)

        result = self.api.template_add_user(template_id, obj)

        self.assertEqual(result.__class__.__name__, response_class)
        self.assertEqual(result, expected)

    def test_template_create_embedded_draft(self):
        request_class = 'TemplateCreateEmbeddedDraftRequest'
        request_data = get_fixture_data(request_class)['default']

        response_class = 'TemplateCreateEmbeddedDraftResponse'
        response_data = get_fixture_data(response_class)['default']

        self.mock_pool.expect_request(
            content_type='multipart/form-data',
            data=request_data,
            response=response_data
        )
        expected = m.TemplateCreateEmbeddedDraftResponse.init(response_data)
        obj = m.TemplateCreateEmbeddedDraftRequest.init(request_data)
        obj.files = [open(f'{get_base_path()}/pdf-sample.pdf', 'rb')]

        result = self.api.template_create_embedded_draft(obj)

        self.assertEqual(result.__class__.__name__, response_class)
        self.assertEqual(result, expected)

        obj.files[0].close()

    def test_template_delete(self):
        self.skipTest('skipping test_template_delete')

    def test_template_files(self):
        self.skipTest('skipping test_template_files')

    def test_template_get(self):
        template_id = 'f57db65d3f933b5316d398057a36176831451a35'

        response_class = 'TemplateGetResponse'
        response_data = get_fixture_data(response_class)['default']

        self.mock_pool.expect_request(
            content_type='application/json',
            response=response_data
        )
        expected = m.TemplateGetResponse.init(response_data)

        result = self.api.template_get(template_id)

        self.assertEqual(result.__class__.__name__, response_class)
        self.assertEqual(result, expected)

    def test_template_list(self):
        account_id = 'all'

        response_class = 'TemplateListResponse'
        response_data = get_fixture_data(response_class)['default']

        self.mock_pool.expect_request(
            content_type='application/json',
            response=response_data
        )
        expected = m.TemplateListResponse.init(response_data)

        result = self.api.template_list(account_id=account_id)

        self.assertEqual(result.__class__.__name__, response_class)
        self.assertEqual(result, expected)

    def test_template_remove_user(self):
        template_id = '21f920ec2b7f4b6bb64d3ed79f26303843046536'

        request_class = 'TemplateRemoveUserRequest'
        request_data = get_fixture_data(request_class)['default']

        response_class = 'TemplateGetResponse'
        response_data = get_fixture_data(response_class)['default']

        self.mock_pool.expect_request(
            content_type='application/json',
            data=request_data,
            response=response_data
        )
        expected = m.TemplateGetResponse.init(response_data)
        obj = m.TemplateRemoveUserRequest.init(request_data)

        result = self.api.template_remove_user(template_id, obj)

        self.assertEqual(result.__class__.__name__, response_class)
        self.assertEqual(result, expected)

    def test_template_update_files(self):
        template_id = '21f920ec2b7f4b6bb64d3ed79f26303843046536'

        request_class = 'TemplateUpdateFilesRequest'
        request_data = get_fixture_data(request_class)['default']

        response_class = 'TemplateUpdateFilesResponse'
        response_data = get_fixture_data(response_class)['default']

        self.mock_pool.expect_request(
            content_type='multipart/form-data',
            data=request_data,
            response=response_data
        )
        expected = m.TemplateUpdateFilesResponse.init(response_data)
        obj = m.TemplateUpdateFilesRequest.init(request_data)
        obj.files = [open(f'{get_base_path()}/pdf-sample.pdf', 'rb')]

        result = self.api.template_update_files(template_id, obj)

        self.assertEqual(result.__class__.__name__, response_class)
        self.assertEqual(result, expected)

        obj.files[0].close()


if __name__ == '__main__':
    unittest.main()



import unittest
import random

from dropbox_sign import ApiClient, ApiException, Configuration, apis, models as m
from test_utils import get_fixture_data, MockPoolManager


class TestAccountApi(unittest.TestCase):
    def setUp(self):
        self.configuration = Configuration()
        self.api_client = ApiClient(self.configuration)
        self.mock_pool = MockPoolManager(self)
        self.api_client.rest_client.pool_manager = self.mock_pool

        self.api = apis.AccountApi(self.api_client)

    def test_http_code_range(self):
        request_class = 'AccountCreateRequest'
        request_data = get_fixture_data(request_class)['default']

        response_class = 'ErrorResponse'
        response_data = get_fixture_data(response_class)['default']

        code = random.randrange(400, 499)

        self.mock_pool.expect_request(
            content_type='application/json',
            data=request_data,
            response=response_data,
            status=code,
        )
        expected = m.ErrorResponse.init(response_data)
        obj = m.AccountCreateRequest.init(request_data)

        try:
            self.api.account_create(obj)
        except ApiException as e:
            self.assertEqual(e.body.__class__.__name__, response_class)
            self.assertEqual(e.body, expected)

    def test_account_create(self):
        request_class = 'AccountCreateRequest'
        request_data = get_fixture_data(request_class)['default']

        response_class = 'AccountCreateResponse'
        response_data = get_fixture_data(response_class)['default']

        self.mock_pool.expect_request(
            content_type='application/json',
            data=request_data,
            response=response_data
        )
        expected = m.AccountCreateResponse.init(response_data)
        obj = m.AccountCreateRequest.init(request_data)

        result = self.api.account_create(obj)

        self.assertEqual(result.__class__.__name__, response_class)
        self.assertEqual(result, expected)

    def test_account_get(self):
        response_class = 'AccountGetResponse'
        response_data = get_fixture_data(response_class)['default']

        self.mock_pool.expect_request(
            content_type='application/json',
            response=response_data
        )
        expected = m.AccountGetResponse.init(response_data)

        result = self.api.account_get(email_address="jack@example.com")

        self.assertEqual(result.__class__.__name__, response_class)
        self.assertEqual(result, expected)

    def test_account_update(self):
        request_class = 'AccountUpdateRequest'
        request_data = get_fixture_data(request_class)['default']

        response_class = 'AccountGetResponse'
        response_data = get_fixture_data(response_class)['default']

        self.mock_pool.expect_request(
            content_type='application/json',
            data=request_data,
            response=response_data
        )
        expected = m.AccountGetResponse.init(response_data)
        obj = m.AccountUpdateRequest.init(request_data)

        result = self.api.account_update(obj)

        self.assertEqual(result.__class__.__name__, response_class)
        self.assertEqual(result, expected)

    def test_account_verify(self):
        request_class = 'AccountVerifyRequest'
        request_data = get_fixture_data(request_class)['default']

        response_class = 'AccountVerifyResponse'
        response_data = get_fixture_data(response_class)['default']

        self.mock_pool.expect_request(
            content_type='application/json',
            data=request_data,
            response=response_data
        )
        expected = m.AccountVerifyResponse.init(response_data)
        obj = m.AccountVerifyRequest.init(request_data)

        result = self.api.account_verify(obj)

        self.assertEqual(result.__class__.__name__, response_class)
        self.assertEqual(result, expected)


if __name__ == '__main__':
    unittest.main()



"""
    Dropbox Sign API

    Dropbox Sign v3 API  # noqa: E501

    The version of the OpenAPI document: 3.0.0
    Contact: apisupport@hellosign.com
    Generated by: https://openapi-generator.tech
"""


from __future__ import annotations
import re  # noqa: F401
import sys  # noqa: F401

from dropbox_sign.api_client import ApiClient, ApiException, Endpoint as _Endpoint
from dropbox_sign.model_utils import (  # noqa: F401
    check_allowed_values,
    check_validations,
    date,
    datetime,
    file_type,
    none_type,
    validate_and_convert_types
)
from dropbox_sign.model.bulk_send_job_get_response import BulkSendJobGetResponse
from dropbox_sign.model.bulk_send_job_list_response import BulkSendJobListResponse
from dropbox_sign.model.error_response import ErrorResponse


class BulkSendJobApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client
        self.bulk_send_job_get_endpoint = _Endpoint(
            settings={
                'response_type': (BulkSendJobGetResponse,),
                'auth': [
                    'api_key',
                    'oauth2'
                ],
                'endpoint_path': '/bulk_send_job/{bulk_send_job_id}',
                'operation_id': 'bulk_send_job_get',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'bulk_send_job_id',
                    'page',
                    'page_size',
                ],
                'required': [
                    'bulk_send_job_id',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'bulk_send_job_id':
                        (str,),
                    'page':
                        (int,),
                    'page_size':
                        (int,),
                },
                'attribute_map': {
                    'bulk_send_job_id': 'bulk_send_job_id',
                    'page': 'page',
                    'page_size': 'page_size',
                },
                'location_map': {
                    'bulk_send_job_id': 'path',
                    'page': 'query',
                    'page_size': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.bulk_send_job_list_endpoint = _Endpoint(
            settings={
                'response_type': (BulkSendJobListResponse,),
                'auth': [
                    'api_key',
                    'oauth2'
                ],
                'endpoint_path': '/bulk_send_job/list',
                'operation_id': 'bulk_send_job_list',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'page',
                    'page_size',
                ],
                'required': [],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'page':
                        (int,),
                    'page_size':
                        (int,),
                },
                'attribute_map': {
                    'page': 'page',
                    'page_size': 'page_size',
                },
                'location_map': {
                    'page': 'query',
                    'page_size': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )

    def bulk_send_job_get(
        self,
        bulk_send_job_id,
        **kwargs
    ) -> BulkSendJobGetResponse:
        """Get Bulk Send Job  # noqa: E501

        Returns the status of the BulkSendJob and its SignatureRequests specified by the `bulk_send_job_id` parameter.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.bulk_send_job_get(bulk_send_job_id, async_req=True)
        >>> result = thread.get()

        Args:
            bulk_send_job_id (str): The id of the BulkSendJob to retrieve.

        Keyword Args:
            page (int): Which page number of the BulkSendJob list to return. Defaults to `1`.. [optional] if omitted the server will use the default value of 1
            page_size (int): Number of objects to be returned per page. Must be between `1` and `100`. Default is 20.. [optional] if omitted the server will use the default value of 20
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            async_req (bool): execute request asynchronously

        Returns:
            BulkSendJobGetResponse
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['bulk_send_job_id'] = \
            bulk_send_job_id
        try:
            return self.bulk_send_job_get_endpoint.call_with_http_info(**kwargs)
        except ApiException as e:
            if e.status == 200:
                e.body = self.api_client.deserialize(
                    response=type('obj_dict', (object,), {'data': e.body}),
                    response_type=[BulkSendJobGetResponse],
                    _check_type=True,
                )

                raise e
            range_code = "4XX"[0]
            range_code_left = int(f"{range_code}00")
            range_code_right = int(f"{range_code}99")

            if range_code_left <= e.status <= range_code_right:
                e.body = self.api_client.deserialize(
                    response=type('obj_dict', (object,), {'data': e.body}),
                    response_type=[ErrorResponse],
                    _check_type=True,
                )

                raise e

    def bulk_send_job_list(
        self,
        **kwargs
    ) -> BulkSendJobListResponse:
        """List Bulk Send Jobs  # noqa: E501

        Returns a list of BulkSendJob that you can access.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.bulk_send_job_list(async_req=True)
        >>> result = thread.get()


        Keyword Args:
            page (int): Which page number of the BulkSendJob List to return. Defaults to `1`.. [optional] if omitted the server will use the default value of 1
            page_size (int): Number of objects to be returned per page. Must be between `1` and `100`. Default is 20.. [optional] if omitted the server will use the default value of 20
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            async_req (bool): execute request asynchronously

        Returns:
            BulkSendJobListResponse
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        try:
            return self.bulk_send_job_list_endpoint.call_with_http_info(**kwargs)
        except ApiException as e:
            if e.status == 200:
                e.body = self.api_client.deserialize(
                    response=type('obj_dict', (object,), {'data': e.body}),
                    response_type=[BulkSendJobListResponse],
                    _check_type=True,
                )

                raise e
            range_code = "4XX"[0]
            range_code_left = int(f"{range_code}00")
            range_code_right = int(f"{range_code}99")

            if range_code_left <= e.status <= range_code_right:
                e.body = self.api_client.deserialize(
                    response=type('obj_dict', (object,), {'data': e.body}),
                    response_type=[ErrorResponse],
                    _check_type=True,
                )

                raise e




import json
import unittest

from dropbox_sign import EventCallbackHelper
from dropbox_sign.models import EventCallbackRequest
from test_utils import get_fixture_data


class TestEventCallbackHelper(unittest.TestCase):
    def test_is_valid(self):
        fixture_data = get_fixture_data('EventCallbackHelper')

        api_key = '324e3b0840f065eb51f3fd63231d0d33daa35d4ed10d27718839e81737065782'
        api_key_rev = api_key[::-1]

        for key, value in fixture_data.items():
            obj = EventCallbackRequest.init(value)

            self.assertTrue(EventCallbackHelper.is_valid(api_key, obj))
            self.assertFalse(EventCallbackHelper.is_valid(api_key_rev, obj))


if __name__ == '__main__':
    unittest.main()



from dropbox_sign.models import EventCallbackRequest, EventCallbackRequestEventMetadata
import hmac
import hashlib


class EventCallbackHelper:
    EVENT_TYPE_ACCOUNT_CALLBACK = 'account_callback'

    EVENT_TYPE_APP_CALLBACK = 'app_callback'

    @classmethod
    def is_valid(cls, api_key, event_callback: EventCallbackRequest):  # noqa: E501
        """Verify that a callback came from HelloSign.com

        Args:
            api_key (string):
            event_callback (EventCallbackRequest):
        """
        hashed = hmac.new(
            bytes(api_key, 'utf-8'),
            bytes(f'{event_callback.event.event_time}{event_callback.event.event_type}', 'utf-8'),
            hashlib.sha256
        ).hexdigest()

        return event_callback.event.event_hash == hashed

    @classmethod
    def get_callback_type(cls, event_callback: EventCallbackRequest):
        """Identifies the callback type, one of "account_callback" or "app_callback".
        "app_callback" will always include a value for "reported_for_app_id"

        Args:
            event_callback (EventCallbackRequest):
        """
        metadata: EventCallbackRequestEventMetadata = event_callback.event.event_metadata

        if not metadata.reported_for_app_id:
            return cls.EVENT_TYPE_ACCOUNT_CALLBACK

        return cls.EVENT_TYPE_APP_CALLBACK



"""
    Dropbox Sign API

    Dropbox Sign v3 API  # noqa: E501

    The version of the OpenAPI document: 3.0.0
    Contact: apisupport@hellosign.com
    Generated by: https://openapi-generator.tech
"""


from __future__ import annotations
from typing import TYPE_CHECKING, Optional, List, Dict, Union
import json  # noqa: F401
import re  # noqa: F401
import sys  # noqa: F401

from dropbox_sign import ApiClient
from dropbox_sign.model_utils import (  # noqa: F401
    ApiTypeError,
    ModelComposed,
    ModelNormal,
    ModelSimple,
    cached_property,
    change_keys_js_to_python,
    convert_js_args_to_python_args,
    date,
    datetime,
    file_type,
    none_type,
    validate_get_composed_info,
    OpenApiModel
)
from dropbox_sign.exceptions import ApiAttributeError
if TYPE_CHECKING:
    from dropbox_sign.model.signature_request_response import SignatureRequestResponse
    from dropbox_sign.model.signature_request_response_attachment import SignatureRequestResponseAttachment
    from dropbox_sign.model.signature_request_response_custom_field_base import SignatureRequestResponseCustomFieldBase
    from dropbox_sign.model.signature_request_response_data_base import SignatureRequestResponseDataBase
    from dropbox_sign.model.signature_request_response_signatures import SignatureRequestResponseSignatures


def lazy_import():
    from dropbox_sign.model.signature_request_response import SignatureRequestResponse
    from dropbox_sign.model.signature_request_response_attachment import SignatureRequestResponseAttachment
    from dropbox_sign.model.signature_request_response_custom_field_base import SignatureRequestResponseCustomFieldBase
    from dropbox_sign.model.signature_request_response_data_base import SignatureRequestResponseDataBase
    from dropbox_sign.model.signature_request_response_signatures import SignatureRequestResponseSignatures
    globals()['SignatureRequestResponse'] = SignatureRequestResponse
    globals()['SignatureRequestResponseAttachment'] = SignatureRequestResponseAttachment
    globals()['SignatureRequestResponseCustomFieldBase'] = SignatureRequestResponseCustomFieldBase
    globals()['SignatureRequestResponseDataBase'] = SignatureRequestResponseDataBase
    globals()['SignatureRequestResponseSignatures'] = SignatureRequestResponseSignatures


class BulkSendJobGetResponseSignatureRequests(ModelComposed):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.

    Attributes:
      allowed_values (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          with a capitalized key describing the allowed value and an allowed
          value. These dicts store the allowed enum values.
      attribute_map (dict): The key is attribute name
          and the value is json key in definition.
      discriminator_value_class_map (dict): A dict to go from the discriminator
          variable value to the discriminator class name.
      validations (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          that stores validations for max_length, min_length, max_items,
          min_items, exclusive_maximum, inclusive_maximum, exclusive_minimum,
          inclusive_minimum, and regex.
      additional_properties_type (tuple): A tuple of classes accepted
          as additional properties values.
    """

    allowed_values = {
    }

    validations = {
    }

    @cached_property
    def additional_properties_type():
        """
        This must be a method because a model may have properties that are
        of type self, this must run after the class is loaded
        """
        lazy_import()
        return (bool, date, datetime, dict, float, int, list, str, none_type,)  # noqa: E501

    _nullable = False

    @cached_property
    def openapi_types():
        """
        This must be a method because a model may have properties that are
        of type self, this must run after the class is loaded

        Returns
            openapi_types (dict): The key is attribute name
                and the value is attribute type.
        """
        lazy_import()
        return {
            'test_mode': (bool, none_type,),  # noqa: E501
            'signature_request_id': (str,),  # noqa: E501
            'requester_email_address': (str,),  # noqa: E501
            'title': (str,),  # noqa: E501
            'original_title': (str,),  # noqa: E501
            'subject': (str, none_type,),  # noqa: E501
            'message': (str, none_type,),  # noqa: E501
            'metadata': ({str: (bool, date, datetime, dict, float, int, list, str, none_type)},),  # noqa: E501
            'created_at': (int,),  # noqa: E501
            'expires_at': (int,),  # noqa: E501
            'is_complete': (bool,),  # noqa: E501
            'is_declined': (bool,),  # noqa: E501
            'has_error': (bool,),  # noqa: E501
            'files_url': (str,),  # noqa: E501
            'signing_url': (str, none_type,),  # noqa: E501
            'details_url': (str,),  # noqa: E501
            'cc_email_addresses': ([str],),  # noqa: E501
            'signing_redirect_url': (str, none_type,),  # noqa: E501
            'template_ids': ([str], none_type,),  # noqa: E501
            'custom_fields': ([SignatureRequestResponseCustomFieldBase], none_type,),  # noqa: E501
            'attachments': ([SignatureRequestResponseAttachment], none_type,),  # noqa: E501
            'response_data': ([SignatureRequestResponseDataBase], none_type,),  # noqa: E501
            'signatures': ([SignatureRequestResponseSignatures],),  # noqa: E501
            'bulk_send_job_id': (str,),  # noqa: E501
        }

    @cached_property
    def discriminator():
        return None

    @staticmethod
    def init(data: any) -> BulkSendJobGetResponseSignatureRequests:
        """
        Attempt to instantiate and hydrate a new instance of this class
        """
        try:
            obj_data = json.dumps(data)
        except TypeError:
            obj_data = data

        return ApiClient().deserialize(
            response=type('obj_dict', (object,), {'data': obj_data}),
            response_type=[BulkSendJobGetResponseSignatureRequests],
            _check_type=True,
        )


    attribute_map = {
        'test_mode': 'test_mode',  # noqa: E501
        'signature_request_id': 'signature_request_id',  # noqa: E501
        'requester_email_address': 'requester_email_address',  # noqa: E501
        'title': 'title',  # noqa: E501
        'original_title': 'original_title',  # noqa: E501
        'subject': 'subject',  # noqa: E501
        'message': 'message',  # noqa: E501
        'metadata': 'metadata',  # noqa: E501
        'created_at': 'created_at',  # noqa: E501
        'expires_at': 'expires_at',  # noqa: E501
        'is_complete': 'is_complete',  # noqa: E501
        'is_declined': 'is_declined',  # noqa: E501
        'has_error': 'has_error',  # noqa: E501
        'files_url': 'files_url',  # noqa: E501
        'signing_url': 'signing_url',  # noqa: E501
        'details_url': 'details_url',  # noqa: E501
        'cc_email_addresses': 'cc_email_addresses',  # noqa: E501
        'signing_redirect_url': 'signing_redirect_url',  # noqa: E501
        'template_ids': 'template_ids',  # noqa: E501
        'custom_fields': 'custom_fields',  # noqa: E501
        'attachments': 'attachments',  # noqa: E501
        'response_data': 'response_data',  # noqa: E501
        'signatures': 'signatures',  # noqa: E501
        'bulk_send_job_id': 'bulk_send_job_id',  # noqa: E501
    }

    read_only_vars = {
    }

    @property
    def test_mode(self) -> Optional[bool]:
        return self.get("test_mode")

    @test_mode.setter
    def test_mode(self, value: Optional[bool]):
        setattr(self, "test_mode", value)

    @property
    def signature_request_id(self) -> str:
        return self.get("signature_request_id")

    @signature_request_id.setter
    def signature_request_id(self, value: str):
        setattr(self, "signature_request_id", value)

    @property
    def requester_email_address(self) -> str:
        return self.get("requester_email_address")

    @requester_email_address.setter
    def requester_email_address(self, value: str):
        setattr(self, "requester_email_address", value)

    @property
    def title(self) -> str:
        return self.get("title")

    @title.setter
    def title(self, value: str):
        setattr(self, "title", value)

    @property
    def original_title(self) -> str:
        return self.get("original_title")

    @original_title.setter
    def original_title(self, value: str):
        setattr(self, "original_title", value)

    @property
    def subject(self) -> Optional[str]:
        return self.get("subject")

    @subject.setter
    def subject(self, value: Optional[str]):
        setattr(self, "subject", value)

    @property
    def message(self) -> Optional[str]:
        return self.get("message")

    @message.setter
    def message(self, value: Optional[str]):
        setattr(self, "message", value)

    @property
    def metadata(self) -> Dict[str, Union[bool, date, datetime, dict, float, int, list, str, none_type]]:
        return self.get("metadata")

    @metadata.setter
    def metadata(self, value: Dict[str, Union[bool, date, datetime, dict, float, int, list, str, none_type]]):
        setattr(self, "metadata", value)

    @property
    def created_at(self) -> int:
        return self.get("created_at")

    @created_at.setter
    def created_at(self, value: int):
        setattr(self, "created_at", value)

    @property
    def expires_at(self) -> int:
        return self.get("expires_at")

    @expires_at.setter
    def expires_at(self, value: int):
        setattr(self, "expires_at", value)

    @property
    def is_complete(self) -> bool:
        return self.get("is_complete")

    @is_complete.setter
    def is_complete(self, value: bool):
        setattr(self, "is_complete", value)

    @property
    def is_declined(self) -> bool:
        return self.get("is_declined")

    @is_declined.setter
    def is_declined(self, value: bool):
        setattr(self, "is_declined", value)

    @property
    def has_error(self) -> bool:
        return self.get("has_error")

    @has_error.setter
    def has_error(self, value: bool):
        setattr(self, "has_error", value)

    @property
    def files_url(self) -> str:
        return self.get("files_url")

    @files_url.setter
    def files_url(self, value: str):
        setattr(self, "files_url", value)

    @property
    def signing_url(self) -> Optional[str]:
        return self.get("signing_url")

    @signing_url.setter
    def signing_url(self, value: Optional[str]):
        setattr(self, "signing_url", value)

    @property
    def details_url(self) -> str:
        return self.get("details_url")

    @details_url.setter
    def details_url(self, value: str):
        setattr(self, "details_url", value)

    @property
    def cc_email_addresses(self) -> List[str]:
        return self.get("cc_email_addresses")

    @cc_email_addresses.setter
    def cc_email_addresses(self, value: List[str]):
        setattr(self, "cc_email_addresses", value)

    @property
    def signing_redirect_url(self) -> Optional[str]:
        return self.get("signing_redirect_url")

    @signing_redirect_url.setter
    def signing_redirect_url(self, value: Optional[str]):
        setattr(self, "signing_redirect_url", value)

    @property
    def template_ids(self) -> Optional[List[str]]:
        return self.get("template_ids")

    @template_ids.setter
    def template_ids(self, value: Optional[List[str]]):
        setattr(self, "template_ids", value)

    @property
    def custom_fields(self) -> Optional[List[SignatureRequestResponseCustomFieldBase]]:
        return self.get("custom_fields")

    @custom_fields.setter
    def custom_fields(self, value: Optional[List[SignatureRequestResponseCustomFieldBase]]):
        setattr(self, "custom_fields", value)

    @property
    def attachments(self) -> Optional[List[SignatureRequestResponseAttachment]]:
        return self.get("attachments")

    @attachments.setter
    def attachments(self, value: Optional[List[SignatureRequestResponseAttachment]]):
        setattr(self, "attachments", value)

    @property
    def response_data(self) -> Optional[List[SignatureRequestResponseDataBase]]:
        return self.get("response_data")

    @response_data.setter
    def response_data(self, value: Optional[List[SignatureRequestResponseDataBase]]):
        setattr(self, "response_data", value)

    @property
    def signatures(self) -> List[SignatureRequestResponseSignatures]:
        return self.get("signatures")

    @signatures.setter
    def signatures(self, value: List[SignatureRequestResponseSignatures]):
        setattr(self, "signatures", value)

    @property
    def bulk_send_job_id(self) -> str:
        return self.get("bulk_send_job_id")

    @bulk_send_job_id.setter
    def bulk_send_job_id(self, value: str):
        setattr(self, "bulk_send_job_id", value)

    @classmethod
    @convert_js_args_to_python_args
    def _from_openapi_data(cls, *args, **kwargs):  # noqa: E501
        """BulkSendJobGetResponseSignatureRequests - a model defined in OpenAPI

        Keyword Args:
            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _spec_property_naming (bool): True if the variable names in the input data
                                are serialized names, as specified in the OpenAPI document.
                                False if the variable names in the input data
                                are pythonic names, e.g. snake case (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            _visited_composed_classes (tuple): This stores a tuple of
                                classes that we have traveled through so that
                                if we see that class again we will not use its
                                discriminator again.
                                When traveling through a discriminator, the
                                composed schema that is
                                is traveled through is added to this set.
                                For example if Animal has a discriminator
                                petType and we pass in "Dog", and the class Dog
                                includes Animal, we move through Animal
                                once using the discriminator, and pick Dog.
                                Then in Dog, we will make an instance of the
                                Animal class but this time we won't travel
                                through its discriminator because we passed in
                                _visited_composed_classes = (Animal,)
            test_mode (bool, none_type): Whether this is a test signature request. Test requests have no legal value. Defaults to `false`.. [optional] if omitted the server will use the default value of False  # noqa: E501
            signature_request_id (str): The id of the SignatureRequest.. [optional]  # noqa: E501
            requester_email_address (str): The email address of the initiator of the SignatureRequest.. [optional]  # noqa: E501
            title (str): The title the specified Account uses for the SignatureRequest.. [optional]  # noqa: E501
            original_title (str): Default Label for account.. [optional]  # noqa: E501
            subject (str, none_type): The subject in the email that was initially sent to the signers.. [optional]  # noqa: E501
            message (str, none_type): The custom message in the email that was initially sent to the signers.. [optional]  # noqa: E501
            metadata ({str: (bool, date, datetime, dict, float, int, list, str, none_type)}): The metadata attached to the signature request.. [optional]  # noqa: E501
            created_at (int): Time the signature request was created.. [optional]  # noqa: E501
            expires_at (int): The time when the signature request will expire unsigned signatures. See [Signature Request Expiration Date](https://developers.hellosign.com/docs/signature-request/expiration/) for details.. [optional]  # noqa: E501
            is_complete (bool): Whether or not the SignatureRequest has been fully executed by all signers.. [optional]  # noqa: E501
            is_declined (bool): Whether or not the SignatureRequest has been declined by a signer.. [optional]  # noqa: E501
            has_error (bool): Whether or not an error occurred (either during the creation of the SignatureRequest or during one of the signings).. [optional]  # noqa: E501
            files_url (str): The URL where a copy of the request's documents can be downloaded.. [optional]  # noqa: E501
            signing_url (str, none_type): The URL where a signer, after authenticating, can sign the documents. This should only be used by users with existing Dropbox Sign accounts as they will be required to log in before signing.. [optional]  # noqa: E501
            details_url (str): The URL where the requester and the signers can view the current status of the SignatureRequest.. [optional]  # noqa: E501
            cc_email_addresses ([str]): A list of email addresses that were CCed on the SignatureRequest. They will receive a copy of the final PDF once all the signers have signed.. [optional]  # noqa: E501
            signing_redirect_url (str, none_type): The URL you want the signer redirected to after they successfully sign.. [optional]  # noqa: E501
            template_ids ([str], none_type): Templates IDs used in this SignatureRequest (if any).. [optional]  # noqa: E501
            custom_fields ([SignatureRequestResponseCustomFieldBase], none_type): An array of Custom Field objects containing the name and type of each custom field.  * Text Field uses `SignatureRequestResponseCustomFieldText` * Checkbox Field uses `SignatureRequestResponseCustomFieldCheckbox`. [optional]  # noqa: E501
            attachments ([SignatureRequestResponseAttachment], none_type): Signer attachments.. [optional]  # noqa: E501
            response_data ([SignatureRequestResponseDataBase], none_type): An array of form field objects containing the name, value, and type of each textbox or checkmark field filled in by the signers.. [optional]  # noqa: E501
            signatures ([SignatureRequestResponseSignatures]): An array of signature objects, 1 for each signer.. [optional]  # noqa: E501
            bulk_send_job_id (str): The id of the BulkSendJob.. [optional]  # noqa: E501
        """

        _check_type = kwargs.pop('_check_type', True)
        _spec_property_naming = kwargs.pop('_spec_property_naming', False)
        _path_to_item = kwargs.pop('_path_to_item', ())
        _configuration = kwargs.pop('_configuration', None)
        _visited_composed_classes = kwargs.pop('_visited_composed_classes', ())

        self = super(OpenApiModel, cls).__new__(cls)

        if args:
            raise ApiTypeError(
                "Invalid positional arguments=%s passed to %s. Remove those invalid positional arguments." % (
                    args,
                    self.__class__.__name__,
                ),
                path_to_item=_path_to_item,
                valid_classes=(self.__class__,),
            )

        self._data_store = {}
        self._check_type = _check_type
        self._spec_property_naming = _spec_property_naming
        self._path_to_item = _path_to_item
        self._configuration = _configuration
        self._visited_composed_classes = _visited_composed_classes + (self.__class__,)

        constant_args = {
            '_check_type': _check_type,
            '_path_to_item': _path_to_item,
            '_spec_property_naming': _spec_property_naming,
            '_configuration': _configuration,
            '_visited_composed_classes': self._visited_composed_classes,
        }
        composed_info = validate_get_composed_info(
            constant_args, kwargs, self)
        self._composed_instances = composed_info[0]
        self._var_name_to_model_instances = composed_info[1]
        self._additional_properties_model_instances = composed_info[2]
        discarded_args = composed_info[3]

        for var_name, var_value in kwargs.items():
            if var_name in discarded_args and \
                        self._configuration is not None and \
                        self._configuration.discard_unknown_keys and \
                        self._additional_properties_model_instances:
                # discard variable.
                continue
            setattr(self, var_name, var_value)

        return self

    required_properties = set([
        '_data_store',
        '_check_type',
        '_spec_property_naming',
        '_path_to_item',
        '_configuration',
        '_visited_composed_classes',
        '_composed_instances',
        '_var_name_to_model_instances',
        '_additional_properties_model_instances',
    ])

    @convert_js_args_to_python_args
    def __init__(self, *args, **kwargs):  # noqa: E501
        """BulkSendJobGetResponseSignatureRequests - a model defined in OpenAPI

        Keyword Args:
            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _spec_property_naming (bool): True if the variable names in the input data
                                are serialized names, as specified in the OpenAPI document.
                                False if the variable names in the input data
                                are pythonic names, e.g. snake case (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            _visited_composed_classes (tuple): This stores a tuple of
                                classes that we have traveled through so that
                                if we see that class again we will not use its
                                discriminator again.
                                When traveling through a discriminator, the
                                composed schema that is
                                is traveled through is added to this set.
                                For example if Animal has a discriminator
                                petType and we pass in "Dog", and the class Dog
                                includes Animal, we move through Animal
                                once using the discriminator, and pick Dog.
                                Then in Dog, we will make an instance of the
                                Animal class but this time we won't travel
                                through its discriminator because we passed in
                                _visited_composed_classes = (Animal,)
            test_mode (bool, none_type): Whether this is a test signature request. Test requests have no legal value. Defaults to `false`.. [optional] if omitted the server will use the default value of False  # noqa: E501
            signature_request_id (str): The id of the SignatureRequest.. [optional]  # noqa: E501
            requester_email_address (str): The email address of the initiator of the SignatureRequest.. [optional]  # noqa: E501
            title (str): The title the specified Account uses for the SignatureRequest.. [optional]  # noqa: E501
            original_title (str): Default Label for account.. [optional]  # noqa: E501
            subject (str, none_type): The subject in the email that was initially sent to the signers.. [optional]  # noqa: E501
            message (str, none_type): The custom message in the email that was initially sent to the signers.. [optional]  # noqa: E501
            metadata ({str: (bool, date, datetime, dict, float, int, list, str, none_type)}): The metadata attached to the signature request.. [optional]  # noqa: E501
            created_at (int): Time the signature request was created.. [optional]  # noqa: E501
            expires_at (int): The time when the signature request will expire unsigned signatures. See [Signature Request Expiration Date](https://developers.hellosign.com/docs/signature-request/expiration/) for details.. [optional]  # noqa: E501
            is_complete (bool): Whether or not the SignatureRequest has been fully executed by all signers.. [optional]  # noqa: E501
            is_declined (bool): Whether or not the SignatureRequest has been declined by a signer.. [optional]  # noqa: E501
            has_error (bool): Whether or not an error occurred (either during the creation of the SignatureRequest or during one of the signings).. [optional]  # noqa: E501
            files_url (str): The URL where a copy of the request's documents can be downloaded.. [optional]  # noqa: E501
            signing_url (str, none_type): The URL where a signer, after authenticating, can sign the documents. This should only be used by users with existing Dropbox Sign accounts as they will be required to log in before signing.. [optional]  # noqa: E501
            details_url (str): The URL where the requester and the signers can view the current status of the SignatureRequest.. [optional]  # noqa: E501
            cc_email_addresses ([str]): A list of email addresses that were CCed on the SignatureRequest. They will receive a copy of the final PDF once all the signers have signed.. [optional]  # noqa: E501
            signing_redirect_url (str, none_type): The URL you want the signer redirected to after they successfully sign.. [optional]  # noqa: E501
            template_ids ([str], none_type): Templates IDs used in this SignatureRequest (if any).. [optional]  # noqa: E501
            custom_fields ([SignatureRequestResponseCustomFieldBase], none_type): An array of Custom Field objects containing the name and type of each custom field.  * Text Field uses `SignatureRequestResponseCustomFieldText` * Checkbox Field uses `SignatureRequestResponseCustomFieldCheckbox`. [optional]  # noqa: E501
            attachments ([SignatureRequestResponseAttachment], none_type): Signer attachments.. [optional]  # noqa: E501
            response_data ([SignatureRequestResponseDataBase], none_type): An array of form field objects containing the name, value, and type of each textbox or checkmark field filled in by the signers.. [optional]  # noqa: E501
            signatures ([SignatureRequestResponseSignatures]): An array of signature objects, 1 for each signer.. [optional]  # noqa: E501
            bulk_send_job_id (str): The id of the BulkSendJob.. [optional]  # noqa: E501
        """

        _check_type = kwargs.pop('_check_type', True)
        _spec_property_naming = kwargs.pop('_spec_property_naming', False)
        _path_to_item = kwargs.pop('_path_to_item', ())
        _configuration = kwargs.pop('_configuration', None)
        _visited_composed_classes = kwargs.pop('_visited_composed_classes', ())

        if args:
            raise ApiTypeError(
                "Invalid positional arguments=%s passed to %s. Remove those invalid positional arguments." % (
                    args,
                    self.__class__.__name__,
                ),
                path_to_item=_path_to_item,
                valid_classes=(self.__class__,),
            )

        self._data_store = {}
        self._check_type = _check_type
        self._spec_property_naming = _spec_property_naming
        self._path_to_item = _path_to_item
        self._configuration = _configuration
        self._visited_composed_classes = _visited_composed_classes + (self.__class__,)

        constant_args = {
            '_check_type': _check_type,
            '_path_to_item': _path_to_item,
            '_spec_property_naming': _spec_property_naming,
            '_configuration': _configuration,
            '_visited_composed_classes': self._visited_composed_classes,
        }
        composed_info = validate_get_composed_info(
            constant_args, kwargs, self)
        self._composed_instances = composed_info[0]
        self._var_name_to_model_instances = composed_info[1]
        self._additional_properties_model_instances = composed_info[2]
        discarded_args = composed_info[3]

        for var_name, var_value in kwargs.items():
            if var_name in discarded_args and \
                        self._configuration is not None and \
                        self._configuration.discard_unknown_keys and \
                        self._additional_properties_model_instances:
                # discard variable.
                continue
            setattr(self, var_name, var_value)
            if var_name in self.read_only_vars:
                raise ApiAttributeError(f"`{var_name}` is a read-only attribute. Use `from_openapi_data` to instantiate "
                                     f"class with read only attributes.")

    @cached_property
    def _composed_schemas():
        # we need this here to make our import statements work
        # we must store _composed_schemas in here so the code is only run
        # when we invoke this method. If we kept this at the class
        # level we would get an error because the class level
        # code would be run when this module is imported, and these composed
        # classes don't exist yet because their module has not finished
        # loading
        lazy_import()
        return {
          'anyOf': [
          ],
          'oneOf': [
          ],
        }



# flake8: noqa

"""
    Dropbox Sign API

    Dropbox Sign v3 API  # noqa: E501

    The version of the OpenAPI document: 3.0.0
    Contact: apisupport@hellosign.com
    Generated by: https://openapi-generator.tech
"""


__version__ = "1.1.1"

# import ApiClient
from dropbox_sign.api_client import ApiClient

# import Configuration
from dropbox_sign.configuration import Configuration

# import exceptions
from dropbox_sign.exceptions import OpenApiException
from dropbox_sign.exceptions import ApiAttributeError
from dropbox_sign.exceptions import ApiTypeError
from dropbox_sign.exceptions import ApiValueError
from dropbox_sign.exceptions import ApiKeyError
from dropbox_sign.exceptions import ApiException

from dropbox_sign.event_callback_helper import EventCallbackHelper



"""
    Dropbox Sign API

    Dropbox Sign v3 API  # noqa: E501

    The version of the OpenAPI document: 3.0.0
    Contact: apisupport@hellosign.com
    Generated by: https://openapi-generator.tech
"""


from __future__ import annotations
from typing import TYPE_CHECKING, Optional, List, Dict, Union
import json  # noqa: F401
import re  # noqa: F401
import sys  # noqa: F401

from dropbox_sign import ApiClient
from dropbox_sign.model_utils import (  # noqa: F401
    ApiTypeError,
    ModelComposed,
    ModelNormal,
    ModelSimple,
    cached_property,
    change_keys_js_to_python,
    convert_js_args_to_python_args,
    date,
    datetime,
    file_type,
    none_type,
    validate_get_composed_info,
    OpenApiModel
)
from dropbox_sign.exceptions import ApiAttributeError
if TYPE_CHECKING:
    from dropbox_sign.model.sub_bulk_signer_list import SubBulkSignerList
    from dropbox_sign.model.sub_cc import SubCC
    from dropbox_sign.model.sub_custom_field import SubCustomField


def lazy_import():
    from dropbox_sign.model.sub_bulk_signer_list import SubBulkSignerList
    from dropbox_sign.model.sub_cc import SubCC
    from dropbox_sign.model.sub_custom_field import SubCustomField
    globals()['SubBulkSignerList'] = SubBulkSignerList
    globals()['SubCC'] = SubCC
    globals()['SubCustomField'] = SubCustomField


class SignatureRequestBulkSendWithTemplateRequest(ModelNormal):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.

    Attributes:
      allowed_values (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          with a capitalized key describing the allowed value and an allowed
          value. These dicts store the allowed enum values.
      attribute_map (dict): The key is attribute name
          and the value is json key in definition.
      discriminator_value_class_map (dict): A dict to go from the discriminator
          variable value to the discriminator class name.
      validations (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          that stores validations for max_length, min_length, max_items,
          min_items, exclusive_maximum, inclusive_maximum, exclusive_minimum,
          inclusive_minimum, and regex.
      additional_properties_type (tuple): A tuple of classes accepted
          as additional properties values.
    """

    allowed_values = {
    }

    validations = {
        ('message',): {
            'max_length': 5000,
        },
        ('metadata',): {
        },
        ('subject',): {
            'max_length': 255,
        },
        ('title',): {
            'max_length': 255,
        },
    }

    @cached_property
    def additional_properties_type():
        """
        This must be a method because a model may have properties that are
        of type self, this must run after the class is loaded
        """
        lazy_import()
        return (bool, date, datetime, dict, float, int, list, str, none_type,)  # noqa: E501

    _nullable = False

    @cached_property
    def openapi_types():
        """
        This must be a method because a model may have properties that are
        of type self, this must run after the class is loaded

        Returns
            openapi_types (dict): The key is attribute name
                and the value is attribute type.
        """
        lazy_import()
        return {
            'template_ids': ([str],),  # noqa: E501
            'signer_file': (file_type,),  # noqa: E501
            'signer_list': ([SubBulkSignerList],),  # noqa: E501
            'allow_decline': (bool,),  # noqa: E501
            'ccs': ([SubCC],),  # noqa: E501
            'client_id': (str,),  # noqa: E501
            'custom_fields': ([SubCustomField],),  # noqa: E501
            'message': (str,),  # noqa: E501
            'metadata': ({str: (bool, date, datetime, dict, float, int, list, str, none_type)},),  # noqa: E501
            'signing_redirect_url': (str,),  # noqa: E501
            'subject': (str,),  # noqa: E501
            'test_mode': (bool,),  # noqa: E501
            'title': (str,),  # noqa: E501
        }

    @cached_property
    def discriminator():
        return None

    @staticmethod
    def init(data: any) -> SignatureRequestBulkSendWithTemplateRequest:
        """
        Attempt to instantiate and hydrate a new instance of this class
        """
        try:
            obj_data = json.dumps(data)
        except TypeError:
            obj_data = data

        return ApiClient().deserialize(
            response=type('obj_dict', (object,), {'data': obj_data}),
            response_type=[SignatureRequestBulkSendWithTemplateRequest],
            _check_type=True,
        )

    attribute_map = {
        'template_ids': 'template_ids',  # noqa: E501
        'signer_file': 'signer_file',  # noqa: E501
        'signer_list': 'signer_list',  # noqa: E501
        'allow_decline': 'allow_decline',  # noqa: E501
        'ccs': 'ccs',  # noqa: E501
        'client_id': 'client_id',  # noqa: E501
        'custom_fields': 'custom_fields',  # noqa: E501
        'message': 'message',  # noqa: E501
        'metadata': 'metadata',  # noqa: E501
        'signing_redirect_url': 'signing_redirect_url',  # noqa: E501
        'subject': 'subject',  # noqa: E501
        'test_mode': 'test_mode',  # noqa: E501
        'title': 'title',  # noqa: E501
    }

    read_only_vars = {
    }

    _composed_schemas = {}

    @property
    def template_ids(self) -> List[str]:
        return self.get("template_ids")

    @template_ids.setter
    def template_ids(self, value: List[str]):
        setattr(self, "template_ids", value)

    @property
    def signer_file(self) -> file_type:
        return self.get("signer_file")

    @signer_file.setter
    def signer_file(self, value: file_type):
        setattr(self, "signer_file", value)

    @property
    def signer_list(self) -> List[SubBulkSignerList]:
        return self.get("signer_list")

    @signer_list.setter
    def signer_list(self, value: List[SubBulkSignerList]):
        setattr(self, "signer_list", value)

    @property
    def allow_decline(self) -> bool:
        return self.get("allow_decline")

    @allow_decline.setter
    def allow_decline(self, value: bool):
        setattr(self, "allow_decline", value)

    @property
    def ccs(self) -> List[SubCC]:
        return self.get("ccs")

    @ccs.setter
    def ccs(self, value: List[SubCC]):
        setattr(self, "ccs", value)

    @property
    def client_id(self) -> str:
        return self.get("client_id")

    @client_id.setter
    def client_id(self, value: str):
        setattr(self, "client_id", value)

    @property
    def custom_fields(self) -> List[SubCustomField]:
        return self.get("custom_fields")

    @custom_fields.setter
    def custom_fields(self, value: List[SubCustomField]):
        setattr(self, "custom_fields", value)

    @property
    def message(self) -> str:
        return self.get("message")

    @message.setter
    def message(self, value: str):
        setattr(self, "message", value)

    @property
    def metadata(self) -> Dict[str, Union[bool, date, datetime, dict, float, int, list, str, none_type]]:
        return self.get("metadata")

    @metadata.setter
    def metadata(self, value: Dict[str, Union[bool, date, datetime, dict, float, int, list, str, none_type]]):
        setattr(self, "metadata", value)

    @property
    def signing_redirect_url(self) -> str:
        return self.get("signing_redirect_url")

    @signing_redirect_url.setter
    def signing_redirect_url(self, value: str):
        setattr(self, "signing_redirect_url", value)

    @property
    def subject(self) -> str:
        return self.get("subject")

    @subject.setter
    def subject(self, value: str):
        setattr(self, "subject", value)

    @property
    def test_mode(self) -> bool:
        return self.get("test_mode")

    @test_mode.setter
    def test_mode(self, value: bool):
        setattr(self, "test_mode", value)

    @property
    def title(self) -> str:
        return self.get("title")

    @title.setter
    def title(self, value: str):
        setattr(self, "title", value)

    @classmethod
    @convert_js_args_to_python_args
    def _from_openapi_data(cls, template_ids, *args, **kwargs):  # noqa: E501
        """SignatureRequestBulkSendWithTemplateRequest - a model defined in OpenAPI

        Args:
            template_ids ([str]): Use `template_ids` to create a SignatureRequest from one or more templates, in the order in which the template will be used.

        Keyword Args:
            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _spec_property_naming (bool): True if the variable names in the input data
                                are serialized names, as specified in the OpenAPI document.
                                False if the variable names in the input data
                                are pythonic names, e.g. snake case (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            _visited_composed_classes (tuple): This stores a tuple of
                                classes that we have traveled through so that
                                if we see that class again we will not use its
                                discriminator again.
                                When traveling through a discriminator, the
                                composed schema that is
                                is traveled through is added to this set.
                                For example if Animal has a discriminator
                                petType and we pass in "Dog", and the class Dog
                                includes Animal, we move through Animal
                                once using the discriminator, and pick Dog.
                                Then in Dog, we will make an instance of the
                                Animal class but this time we won't travel
                                through its discriminator because we passed in
                                _visited_composed_classes = (Animal,)
            signer_file (file_type): `signer_file` is a CSV file defining values and options for signer fields. Required unless a `signer_list` is used, you may not use both. The CSV can have the following columns:  - `name`: the name of the signer filling the role of RoleName - `email_address`: email address of the signer filling the role of RoleName - `pin`: the 4- to 12-character access code that will secure this signer's signature page (optional) - `sms_phone_number`: An E.164 formatted phone number that will receive a code via SMS to access this signer's signature page. (optional)      **Note**: Not available in test mode and requires a Standard plan or higher. - `*_field`: any column with a _field\" suffix will be treated as a custom field (optional)      You may only specify field values here, any other options should be set in the custom_fields request parameter.  Example CSV:  ``` name, email_address, pin, company_field George, george@example.com, d79a3td, ABC Corp Mary, mary@example.com, gd9as5b, 123 LLC ```. [optional]  # noqa: E501
            signer_list ([SubBulkSignerList]): `signer_list` is an array defining values and options for signer fields. Required unless a `signer_file` is used, you may not use both.. [optional]  # noqa: E501
            allow_decline (bool): Allows signers to decline to sign a document if `true`. Defaults to `false`.. [optional] if omitted the server will use the default value of False  # noqa: E501
            ccs ([SubCC]): Add CC email recipients. Required when a CC role exists for the Template.. [optional]  # noqa: E501
            client_id (str): The client id of the API App you want to associate with this request. Used to apply the branding and callback url defined for the app.. [optional]  # noqa: E501
            custom_fields ([SubCustomField]): When used together with merge fields, `custom_fields` allows users to add pre-filled data to their signature requests.  Pre-filled data can be used with \"send-once\" signature requests by adding merge fields with `form_fields_per_document` or [Text Tags](https://app.hellosign.com/api/textTagsWalkthrough#TextTagIntro) while passing values back with `custom_fields` together in one API call.  For using pre-filled on repeatable signature requests, merge fields are added to templates in the Dropbox Sign UI or by calling [/template/create_embedded_draft](/api/reference/operation/templateCreateEmbeddedDraft) and then passing `custom_fields` on subsequent signature requests referencing that template.. [optional]  # noqa: E501
            message (str): The custom message in the email that will be sent to the signers.. [optional]  # noqa: E501
            metadata ({str: (bool, date, datetime, dict, float, int, list, str, none_type)}): Key-value data that should be attached to the signature request. This metadata is included in all API responses and events involving the signature request. For example, use the metadata field to store a signer's order number for look up when receiving events for the signature request.  Each request can include up to 10 metadata keys (or 50 nested metadata keys), with key names up to 40 characters long and values up to 1000 characters long.. [optional]  # noqa: E501
            signing_redirect_url (str): The URL you want signers redirected to after they successfully sign.. [optional]  # noqa: E501
            subject (str): The subject in the email that will be sent to the signers.. [optional]  # noqa: E501
            test_mode (bool): Whether this is a test, the signature request will not be legally binding if set to `true`. Defaults to `false`.. [optional] if omitted the server will use the default value of False  # noqa: E501
            title (str): The title you want to assign to the SignatureRequest.. [optional]  # noqa: E501
        """

        _check_type = kwargs.pop('_check_type', True)
        _spec_property_naming = kwargs.pop('_spec_property_naming', False)
        _path_to_item = kwargs.pop('_path_to_item', ())
        _configuration = kwargs.pop('_configuration', None)
        _visited_composed_classes = kwargs.pop('_visited_composed_classes', ())

        self = super(OpenApiModel, cls).__new__(cls)

        if args:
            raise ApiTypeError(
                "Invalid positional arguments=%s passed to %s. Remove those invalid positional arguments." % (
                    args,
                    self.__class__.__name__,
                ),
                path_to_item=_path_to_item,
                valid_classes=(self.__class__,),
            )

        self._data_store = {}
        self._check_type = _check_type
        self._spec_property_naming = _spec_property_naming
        self._path_to_item = _path_to_item
        self._configuration = _configuration
        self._visited_composed_classes = _visited_composed_classes + (self.__class__,)

        self.template_ids = template_ids
        for var_name, var_value in kwargs.items():
            if var_name not in self.attribute_map and \
                        self._configuration is not None and \
                        self._configuration.discard_unknown_keys and \
                        self.additional_properties_type is None:
                # discard variable.
                continue
            setattr(self, var_name, var_value)
        return self

    required_properties = set([
        '_data_store',
        '_check_type',
        '_spec_property_naming',
        '_path_to_item',
        '_configuration',
        '_visited_composed_classes',
    ])

    @convert_js_args_to_python_args
    def __init__(self, template_ids, *args, **kwargs):  # noqa: E501
        """SignatureRequestBulkSendWithTemplateRequest - a model defined in OpenAPI

        Args:
            template_ids ([str]): Use `template_ids` to create a SignatureRequest from one or more templates, in the order in which the template will be used.

        Keyword Args:
            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _spec_property_naming (bool): True if the variable names in the input data
                                are serialized names, as specified in the OpenAPI document.
                                False if the variable names in the input data
                                are pythonic names, e.g. snake case (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            _visited_composed_classes (tuple): This stores a tuple of
                                classes that we have traveled through so that
                                if we see that class again we will not use its
                                discriminator again.
                                When traveling through a discriminator, the
                                composed schema that is
                                is traveled through is added to this set.
                                For example if Animal has a discriminator
                                petType and we pass in "Dog", and the class Dog
                                includes Animal, we move through Animal
                                once using the discriminator, and pick Dog.
                                Then in Dog, we will make an instance of the
                                Animal class but this time we won't travel
                                through its discriminator because we passed in
                                _visited_composed_classes = (Animal,)
            signer_file (file_type): `signer_file` is a CSV file defining values and options for signer fields. Required unless a `signer_list` is used, you may not use both. The CSV can have the following columns:  - `name`: the name of the signer filling the role of RoleName - `email_address`: email address of the signer filling the role of RoleName - `pin`: the 4- to 12-character access code that will secure this signer's signature page (optional) - `sms_phone_number`: An E.164 formatted phone number that will receive a code via SMS to access this signer's signature page. (optional)      **Note**: Not available in test mode and requires a Standard plan or higher. - `*_field`: any column with a _field\" suffix will be treated as a custom field (optional)      You may only specify field values here, any other options should be set in the custom_fields request parameter.  Example CSV:  ``` name, email_address, pin, company_field George, george@example.com, d79a3td, ABC Corp Mary, mary@example.com, gd9as5b, 123 LLC ```. [optional]  # noqa: E501
            signer_list ([SubBulkSignerList]): `signer_list` is an array defining values and options for signer fields. Required unless a `signer_file` is used, you may not use both.. [optional]  # noqa: E501
            allow_decline (bool): Allows signers to decline to sign a document if `true`. Defaults to `false`.. [optional] if omitted the server will use the default value of False  # noqa: E501
            ccs ([SubCC]): Add CC email recipients. Required when a CC role exists for the Template.. [optional]  # noqa: E501
            client_id (str): The client id of the API App you want to associate with this request. Used to apply the branding and callback url defined for the app.. [optional]  # noqa: E501
            custom_fields ([SubCustomField]): When used together with merge fields, `custom_fields` allows users to add pre-filled data to their signature requests.  Pre-filled data can be used with \"send-once\" signature requests by adding merge fields with `form_fields_per_document` or [Text Tags](https://app.hellosign.com/api/textTagsWalkthrough#TextTagIntro) while passing values back with `custom_fields` together in one API call.  For using pre-filled on repeatable signature requests, merge fields are added to templates in the Dropbox Sign UI or by calling [/template/create_embedded_draft](/api/reference/operation/templateCreateEmbeddedDraft) and then passing `custom_fields` on subsequent signature requests referencing that template.. [optional]  # noqa: E501
            message (str): The custom message in the email that will be sent to the signers.. [optional]  # noqa: E501
            metadata ({str: (bool, date, datetime, dict, float, int, list, str, none_type)}): Key-value data that should be attached to the signature request. This metadata is included in all API responses and events involving the signature request. For example, use the metadata field to store a signer's order number for look up when receiving events for the signature request.  Each request can include up to 10 metadata keys (or 50 nested metadata keys), with key names up to 40 characters long and values up to 1000 characters long.. [optional]  # noqa: E501
            signing_redirect_url (str): The URL you want signers redirected to after they successfully sign.. [optional]  # noqa: E501
            subject (str): The subject in the email that will be sent to the signers.. [optional]  # noqa: E501
            test_mode (bool): Whether this is a test, the signature request will not be legally binding if set to `true`. Defaults to `false`.. [optional] if omitted the server will use the default value of False  # noqa: E501
            title (str): The title you want to assign to the SignatureRequest.. [optional]  # noqa: E501
        """

        _check_type = kwargs.pop('_check_type', True)
        _spec_property_naming = kwargs.pop('_spec_property_naming', False)
        _path_to_item = kwargs.pop('_path_to_item', ())
        _configuration = kwargs.pop('_configuration', None)
        _visited_composed_classes = kwargs.pop('_visited_composed_classes', ())

        if args:
            raise ApiTypeError(
                "Invalid positional arguments=%s passed to %s. Remove those invalid positional arguments." % (
                    args,
                    self.__class__.__name__,
                ),
                path_to_item=_path_to_item,
                valid_classes=(self.__class__,),
            )

        self._data_store = {}
        self._check_type = _check_type
        self._spec_property_naming = _spec_property_naming
        self._path_to_item = _path_to_item
        self._configuration = _configuration
        self._visited_composed_classes = _visited_composed_classes + (self.__class__,)

        self.template_ids = template_ids
        for var_name, var_value in kwargs.items():
            if var_name not in self.attribute_map and \
                        self._configuration is not None and \
                        self._configuration.discard_unknown_keys and \
                        self.additional_properties_type is None:
                # discard variable.
                continue
            setattr(self, var_name, var_value)
            if var_name in self.read_only_vars:
                raise ApiAttributeError(f"`{var_name}` is a read-only attribute. Use `from_openapi_data` to instantiate "
                                     f"class with read only attributes.")



import os
import json
import urllib3


def get_base_path():
    return os.path.dirname(os.path.abspath(__file__)) + f'/../test_fixtures'


def get_fixture_data(filename):
    path = get_base_path() + f'/{filename}.json'

    file = open(path, 'r')
    fixture_data = json.load(file)
    file.close()

    return fixture_data


class MockPoolManager(object):
    def __init__(self, tc):
        self._tc = tc
        self._expected_request = {}
        self._expected_response = {}
        self._request_fields = []

    def expect_request(self, content_type, data=None, response=None, status=200):
        self._expected_request = {
            'content_type': content_type,
            'data': data,
        }
        self._expected_response = {
            'data': response,
            'status': status,
        }

    def request(self, *args, **kwargs):
        if self._expected_request['content_type'] == 'multipart/form-data':
            self._tc.assertTrue('encode_multipart' in kwargs.keys())
            self._tc.assertTrue(kwargs['encode_multipart'])
        else:
            self._tc.assertFalse('encode_multipart' in kwargs.keys())

        if 'fields' in kwargs:
            self._request_fields = kwargs['fields']

        return urllib3.HTTPResponse(
            status=self._expected_response['status'],
            preload_content=True,
            body=json.dumps(self._expected_response['data']).encode()
        )

    def get_fields(self):
        return self._request_fields



"""
    Dropbox Sign API

    Dropbox Sign v3 API  # noqa: E501

    The version of the OpenAPI document: 3.0.0
    Contact: apisupport@hellosign.com
    Generated by: https://openapi-generator.tech
"""


from __future__ import annotations
from typing import TYPE_CHECKING, Optional, List, Dict, Union
import json  # noqa: F401
import re  # noqa: F401
import sys  # noqa: F401

from dropbox_sign import ApiClient
from dropbox_sign.model_utils import (  # noqa: F401
    ApiTypeError,
    ModelComposed,
    ModelNormal,
    ModelSimple,
    cached_property,
    change_keys_js_to_python,
    convert_js_args_to_python_args,
    date,
    datetime,
    file_type,
    none_type,
    validate_get_composed_info,
    OpenApiModel
)
from dropbox_sign.exceptions import ApiAttributeError
if TYPE_CHECKING:
    from dropbox_sign.model.sub_attachment import SubAttachment
    from dropbox_sign.model.sub_custom_field import SubCustomField
    from dropbox_sign.model.sub_editor_options import SubEditorOptions
    from dropbox_sign.model.sub_field_options import SubFieldOptions
    from dropbox_sign.model.sub_form_field_group import SubFormFieldGroup
    from dropbox_sign.model.sub_form_field_rule import SubFormFieldRule
    from dropbox_sign.model.sub_form_fields_per_document_base import SubFormFieldsPerDocumentBase
    from dropbox_sign.model.sub_signing_options import SubSigningOptions
    from dropbox_sign.model.sub_unclaimed_draft_signer import SubUnclaimedDraftSigner


def lazy_import():
    from dropbox_sign.model.sub_attachment import SubAttachment
    from dropbox_sign.model.sub_custom_field import SubCustomField
    from dropbox_sign.model.sub_editor_options import SubEditorOptions
    from dropbox_sign.model.sub_field_options import SubFieldOptions
    from dropbox_sign.model.sub_form_field_group import SubFormFieldGroup
    from dropbox_sign.model.sub_form_field_rule import SubFormFieldRule
    from dropbox_sign.model.sub_form_fields_per_document_base import SubFormFieldsPerDocumentBase
    from dropbox_sign.model.sub_signing_options import SubSigningOptions
    from dropbox_sign.model.sub_unclaimed_draft_signer import SubUnclaimedDraftSigner
    globals()['SubAttachment'] = SubAttachment
    globals()['SubCustomField'] = SubCustomField
    globals()['SubEditorOptions'] = SubEditorOptions
    globals()['SubFieldOptions'] = SubFieldOptions
    globals()['SubFormFieldGroup'] = SubFormFieldGroup
    globals()['SubFormFieldRule'] = SubFormFieldRule
    globals()['SubFormFieldsPerDocumentBase'] = SubFormFieldsPerDocumentBase
    globals()['SubSigningOptions'] = SubSigningOptions
    globals()['SubUnclaimedDraftSigner'] = SubUnclaimedDraftSigner


class UnclaimedDraftCreateEmbeddedRequest(ModelNormal):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.

    Attributes:
      allowed_values (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          with a capitalized key describing the allowed value and an allowed
          value. These dicts store the allowed enum values.
      attribute_map (dict): The key is attribute name
          and the value is json key in definition.
      discriminator_value_class_map (dict): A dict to go from the discriminator
          variable value to the discriminator class name.
      validations (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          that stores validations for max_length, min_length, max_items,
          min_items, exclusive_maximum, inclusive_maximum, exclusive_minimum,
          inclusive_minimum, and regex.
      additional_properties_type (tuple): A tuple of classes accepted
          as additional properties values.
    """

    allowed_values = {
        ('type',): {
            'SEND_DOCUMENT': "send_document",
            'REQUEST_SIGNATURE': "request_signature",
        },
    }

    validations = {
        ('message',): {
            'max_length': 5000,
        },
        ('metadata',): {
        },
        ('subject',): {
            'max_length': 200,
        },
    }

    @cached_property
    def additional_properties_type():
        """
        This must be a method because a model may have properties that are
        of type self, this must run after the class is loaded
        """
        lazy_import()
        return (bool, date, datetime, dict, float, int, list, str, none_type,)  # noqa: E501

    _nullable = False

    @cached_property
    def openapi_types():
        """
        This must be a method because a model may have properties that are
        of type self, this must run after the class is loaded

        Returns
            openapi_types (dict): The key is attribute name
                and the value is attribute type.
        """
        lazy_import()
        return {
            'client_id': (str,),  # noqa: E501
            'requester_email_address': (str,),  # noqa: E501
            'files': ([file_type],),  # noqa: E501
            'file_urls': ([str],),  # noqa: E501
            'allow_ccs': (bool,),  # noqa: E501
            'allow_decline': (bool,),  # noqa: E501
            'allow_reassign': (bool,),  # noqa: E501
            'attachments': ([SubAttachment],),  # noqa: E501
            'cc_email_addresses': ([str],),  # noqa: E501
            'custom_fields': ([SubCustomField],),  # noqa: E501
            'editor_options': (SubEditorOptions,),  # noqa: E501
            'field_options': (SubFieldOptions,),  # noqa: E501
            'force_signer_page': (bool,),  # noqa: E501
            'force_subject_message': (bool,),  # noqa: E501
            'form_field_groups': ([SubFormFieldGroup],),  # noqa: E501
            'form_field_rules': ([SubFormFieldRule],),  # noqa: E501
            'form_fields_per_document': ([SubFormFieldsPerDocumentBase],),  # noqa: E501
            'hide_text_tags': (bool,),  # noqa: E501
            'hold_request': (bool,),  # noqa: E501
            'is_for_embedded_signing': (bool,),  # noqa: E501
            'message': (str,),  # noqa: E501
            'metadata': ({str: (bool, date, datetime, dict, float, int, list, str, none_type)},),  # noqa: E501
            'requesting_redirect_url': (str,),  # noqa: E501
            'show_preview': (bool,),  # noqa: E501
            'show_progress_stepper': (bool,),  # noqa: E501
            'signers': ([SubUnclaimedDraftSigner],),  # noqa: E501
            'signing_options': (SubSigningOptions,),  # noqa: E501
            'signing_redirect_url': (str,),  # noqa: E501
            'skip_me_now': (bool,),  # noqa: E501
            'subject': (str,),  # noqa: E501
            'test_mode': (bool,),  # noqa: E501
            'type': (str,),  # noqa: E501
            'use_preexisting_fields': (bool,),  # noqa: E501
            'use_text_tags': (bool,),  # noqa: E501
            'populate_auto_fill_fields': (bool,),  # noqa: E501
            'expires_at': (int, none_type,),  # noqa: E501
        }

    @cached_property
    def discriminator():
        return None

    @staticmethod
    def init(data: any) -> UnclaimedDraftCreateEmbeddedRequest:
        """
        Attempt to instantiate and hydrate a new instance of this class
        """
        try:
            obj_data = json.dumps(data)
        except TypeError:
            obj_data = data

        return ApiClient().deserialize(
            response=type('obj_dict', (object,), {'data': obj_data}),
            response_type=[UnclaimedDraftCreateEmbeddedRequest],
            _check_type=True,
        )

    attribute_map = {
        'client_id': 'client_id',  # noqa: E501
        'requester_email_address': 'requester_email_address',  # noqa: E501
        'files': 'files',  # noqa: E501
        'file_urls': 'file_urls',  # noqa: E501
        'allow_ccs': 'allow_ccs',  # noqa: E501
        'allow_decline': 'allow_decline',  # noqa: E501
        'allow_reassign': 'allow_reassign',  # noqa: E501
        'attachments': 'attachments',  # noqa: E501
        'cc_email_addresses': 'cc_email_addresses',  # noqa: E501
        'custom_fields': 'custom_fields',  # noqa: E501
        'editor_options': 'editor_options',  # noqa: E501
        'field_options': 'field_options',  # noqa: E501
        'force_signer_page': 'force_signer_page',  # noqa: E501
        'force_subject_message': 'force_subject_message',  # noqa: E501
        'form_field_groups': 'form_field_groups',  # noqa: E501
        'form_field_rules': 'form_field_rules',  # noqa: E501
        'form_fields_per_document': 'form_fields_per_document',  # noqa: E501
        'hide_text_tags': 'hide_text_tags',  # noqa: E501
        'hold_request': 'hold_request',  # noqa: E501
        'is_for_embedded_signing': 'is_for_embedded_signing',  # noqa: E501
        'message': 'message',  # noqa: E501
        'metadata': 'metadata',  # noqa: E501
        'requesting_redirect_url': 'requesting_redirect_url',  # noqa: E501
        'show_preview': 'show_preview',  # noqa: E501
        'show_progress_stepper': 'show_progress_stepper',  # noqa: E501
        'signers': 'signers',  # noqa: E501
        'signing_options': 'signing_options',  # noqa: E501
        'signing_redirect_url': 'signing_redirect_url',  # noqa: E501
        'skip_me_now': 'skip_me_now',  # noqa: E501
        'subject': 'subject',  # noqa: E501
        'test_mode': 'test_mode',  # noqa: E501
        'type': 'type',  # noqa: E501
        'use_preexisting_fields': 'use_preexisting_fields',  # noqa: E501
        'use_text_tags': 'use_text_tags',  # noqa: E501
        'populate_auto_fill_fields': 'populate_auto_fill_fields',  # noqa: E501
        'expires_at': 'expires_at',  # noqa: E501
    }

    read_only_vars = {
    }

    _composed_schemas = {}

    @property
    def client_id(self) -> str:
        return self.get("client_id")

    @client_id.setter
    def client_id(self, value: str):
        setattr(self, "client_id", value)

    @property
    def requester_email_address(self) -> str:
        return self.get("requester_email_address")

    @requester_email_address.setter
    def requester_email_address(self, value: str):
        setattr(self, "requester_email_address", value)

    @property
    def files(self) -> List[file_type]:
        return self.get("files")

    @files.setter
    def files(self, value: List[file_type]):
        setattr(self, "files", value)

    @property
    def file_urls(self) -> List[str]:
        return self.get("file_urls")

    @file_urls.setter
    def file_urls(self, value: List[str]):
        setattr(self, "file_urls", value)

    @property
    def allow_ccs(self) -> bool:
        return self.get("allow_ccs")

    @allow_ccs.setter
    def allow_ccs(self, value: bool):
        setattr(self, "allow_ccs", value)

    @property
    def allow_decline(self) -> bool:
        return self.get("allow_decline")

    @allow_decline.setter
    def allow_decline(self, value: bool):
        setattr(self, "allow_decline", value)

    @property
    def allow_reassign(self) -> bool:
        return self.get("allow_reassign")

    @allow_reassign.setter
    def allow_reassign(self, value: bool):
        setattr(self, "allow_reassign", value)

    @property
    def attachments(self) -> List[SubAttachment]:
        return self.get("attachments")

    @attachments.setter
    def attachments(self, value: List[SubAttachment]):
        setattr(self, "attachments", value)

    @property
    def cc_email_addresses(self) -> List[str]:
        return self.get("cc_email_addresses")

    @cc_email_addresses.setter
    def cc_email_addresses(self, value: List[str]):
        setattr(self, "cc_email_addresses", value)

    @property
    def custom_fields(self) -> List[SubCustomField]:
        return self.get("custom_fields")

    @custom_fields.setter
    def custom_fields(self, value: List[SubCustomField]):
        setattr(self, "custom_fields", value)

    @property
    def editor_options(self) -> SubEditorOptions:
        return self.get("editor_options")

    @editor_options.setter
    def editor_options(self, value: SubEditorOptions):
        setattr(self, "editor_options", value)

    @property
    def field_options(self) -> SubFieldOptions:
        return self.get("field_options")

    @field_options.setter
    def field_options(self, value: SubFieldOptions):
        setattr(self, "field_options", value)

    @property
    def force_signer_page(self) -> bool:
        return self.get("force_signer_page")

    @force_signer_page.setter
    def force_signer_page(self, value: bool):
        setattr(self, "force_signer_page", value)

    @property
    def force_subject_message(self) -> bool:
        return self.get("force_subject_message")

    @force_subject_message.setter
    def force_subject_message(self, value: bool):
        setattr(self, "force_subject_message", value)

    @property
    def form_field_groups(self) -> List[SubFormFieldGroup]:
        return self.get("form_field_groups")

    @form_field_groups.setter
    def form_field_groups(self, value: List[SubFormFieldGroup]):
        setattr(self, "form_field_groups", value)

    @property
    def form_field_rules(self) -> List[SubFormFieldRule]:
        return self.get("form_field_rules")

    @form_field_rules.setter
    def form_field_rules(self, value: List[SubFormFieldRule]):
        setattr(self, "form_field_rules", value)

    @property
    def form_fields_per_document(self) -> List[SubFormFieldsPerDocumentBase]:
        return self.get("form_fields_per_document")

    @form_fields_per_document.setter
    def form_fields_per_document(self, value: List[SubFormFieldsPerDocumentBase]):
        setattr(self, "form_fields_per_document", value)

    @property
    def hide_text_tags(self) -> bool:
        return self.get("hide_text_tags")

    @hide_text_tags.setter
    def hide_text_tags(self, value: bool):
        setattr(self, "hide_text_tags", value)

    @property
    def hold_request(self) -> bool:
        return self.get("hold_request")

    @hold_request.setter
    def hold_request(self, value: bool):
        setattr(self, "hold_request", value)

    @property
    def is_for_embedded_signing(self) -> bool:
        return self.get("is_for_embedded_signing")

    @is_for_embedded_signing.setter
    def is_for_embedded_signing(self, value: bool):
        setattr(self, "is_for_embedded_signing", value)

    @property
    def message(self) -> str:
        return self.get("message")

    @message.setter
    def message(self, value: str):
        setattr(self, "message", value)

    @property
    def metadata(self) -> Dict[str, Union[bool, date, datetime, dict, float, int, list, str, none_type]]:
        return self.get("metadata")

    @metadata.setter
    def metadata(self, value: Dict[str, Union[bool, date, datetime, dict, float, int, list, str, none_type]]):
        setattr(self, "metadata", value)

    @property
    def requesting_redirect_url(self) -> str:
        return self.get("requesting_redirect_url")

    @requesting_redirect_url.setter
    def requesting_redirect_url(self, value: str):
        setattr(self, "requesting_redirect_url", value)

    @property
    def show_preview(self) -> bool:
        return self.get("show_preview")

    @show_preview.setter
    def show_preview(self, value: bool):
        setattr(self, "show_preview", value)

    @property
    def show_progress_stepper(self) -> bool:
        return self.get("show_progress_stepper")

    @show_progress_stepper.setter
    def show_progress_stepper(self, value: bool):
        setattr(self, "show_progress_stepper", value)

    @property
    def signers(self) -> List[SubUnclaimedDraftSigner]:
        return self.get("signers")

    @signers.setter
    def signers(self, value: List[SubUnclaimedDraftSigner]):
        setattr(self, "signers", value)

    @property
    def signing_options(self) -> SubSigningOptions:
        return self.get("signing_options")

    @signing_options.setter
    def signing_options(self, value: SubSigningOptions):
        setattr(self, "signing_options", value)

    @property
    def signing_redirect_url(self) -> str:
        return self.get("signing_redirect_url")

    @signing_redirect_url.setter
    def signing_redirect_url(self, value: str):
        setattr(self, "signing_redirect_url", value)

    @property
    def skip_me_now(self) -> bool:
        return self.get("skip_me_now")

    @skip_me_now.setter
    def skip_me_now(self, value: bool):
        setattr(self, "skip_me_now", value)

    @property
    def subject(self) -> str:
        return self.get("subject")

    @subject.setter
    def subject(self, value: str):
        setattr(self, "subject", value)

    @property
    def test_mode(self) -> bool:
        return self.get("test_mode")

    @test_mode.setter
    def test_mode(self, value: bool):
        setattr(self, "test_mode", value)

    @property
    def type(self) -> str:
        return self.get("type")

    @type.setter
    def type(self, value: str):
        setattr(self, "type", value)

    @property
    def use_preexisting_fields(self) -> bool:
        return self.get("use_preexisting_fields")

    @use_preexisting_fields.setter
    def use_preexisting_fields(self, value: bool):
        setattr(self, "use_preexisting_fields", value)

    @property
    def use_text_tags(self) -> bool:
        return self.get("use_text_tags")

    @use_text_tags.setter
    def use_text_tags(self, value: bool):
        setattr(self, "use_text_tags", value)

    @property
    def populate_auto_fill_fields(self) -> bool:
        return self.get("populate_auto_fill_fields")

    @populate_auto_fill_fields.setter
    def populate_auto_fill_fields(self, value: bool):
        setattr(self, "populate_auto_fill_fields", value)

    @property
    def expires_at(self) -> Optional[int]:
        return self.get("expires_at")

    @expires_at.setter
    def expires_at(self, value: Optional[int]):
        setattr(self, "expires_at", value)

    @classmethod
    @convert_js_args_to_python_args
    def _from_openapi_data(cls, client_id, requester_email_address, *args, **kwargs):  # noqa: E501
        """UnclaimedDraftCreateEmbeddedRequest - a model defined in OpenAPI

        Args:
            client_id (str): Client id of the app used to create the draft. Used to apply the branding and callback url defined for the app.
            requester_email_address (str): The email address of the user that should be designated as the requester of this draft, if the draft type is `request_signature`.

        Keyword Args:
            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _spec_property_naming (bool): True if the variable names in the input data
                                are serialized names, as specified in the OpenAPI document.
                                False if the variable names in the input data
                                are pythonic names, e.g. snake case (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            _visited_composed_classes (tuple): This stores a tuple of
                                classes that we have traveled through so that
                                if we see that class again we will not use its
                                discriminator again.
                                When traveling through a discriminator, the
                                composed schema that is
                                is traveled through is added to this set.
                                For example if Animal has a discriminator
                                petType and we pass in "Dog", and the class Dog
                                includes Animal, we move through Animal
                                once using the discriminator, and pick Dog.
                                Then in Dog, we will make an instance of the
                                Animal class but this time we won't travel
                                through its discriminator because we passed in
                                _visited_composed_classes = (Animal,)
            files ([file_type]): Use `files[]` to indicate the uploaded file(s) to send for signature.  This endpoint requires either **files** or **file_urls[]**, but not both.. [optional]  # noqa: E501
            file_urls ([str]): Use `file_urls[]` to have Dropbox Sign download the file(s) to send for signature.  This endpoint requires either **files** or **file_urls[]**, but not both.. [optional]  # noqa: E501
            allow_ccs (bool): This allows the requester to specify whether the user is allowed to provide email addresses to CC when claiming the draft.. [optional] if omitted the server will use the default value of True  # noqa: E501
            allow_decline (bool): Allows signers to decline to sign a document if `true`. Defaults to `false`.. [optional] if omitted the server will use the default value of False  # noqa: E501
            allow_reassign (bool): Allows signers to reassign their signature requests to other signers if set to `true`. Defaults to `false`.  **Note**: Only available for Premium plan and higher.. [optional] if omitted the server will use the default value of False  # noqa: E501
            attachments ([SubAttachment]): A list describing the attachments. [optional]  # noqa: E501
            cc_email_addresses ([str]): The email addresses that should be CCed.. [optional]  # noqa: E501
            custom_fields ([SubCustomField]): When used together with merge fields, `custom_fields` allows users to add pre-filled data to their signature requests.  Pre-filled data can be used with \"send-once\" signature requests by adding merge fields with `form_fields_per_document` or [Text Tags](https://app.hellosign.com/api/textTagsWalkthrough#TextTagIntro) while passing values back with `custom_fields` together in one API call.  For using pre-filled on repeatable signature requests, merge fields are added to templates in the Dropbox Sign UI or by calling [/template/create_embedded_draft](/api/reference/operation/templateCreateEmbeddedDraft) and then passing `custom_fields` on subsequent signature requests referencing that template.. [optional]  # noqa: E501
            editor_options (SubEditorOptions): [optional]  # noqa: E501
            field_options (SubFieldOptions): [optional]  # noqa: E501
            force_signer_page (bool): Provide users the ability to review/edit the signers.. [optional] if omitted the server will use the default value of False  # noqa: E501
            force_subject_message (bool): Provide users the ability to review/edit the subject and message.. [optional] if omitted the server will use the default value of False  # noqa: E501
            form_field_groups ([SubFormFieldGroup]): Group information for fields defined in `form_fields_per_document`. String-indexed JSON array with `group_label` and `requirement` keys. `form_fields_per_document` must contain fields referencing a group defined in `form_field_groups`.. [optional]  # noqa: E501
            form_field_rules ([SubFormFieldRule]): Conditional Logic rules for fields defined in `form_fields_per_document`.. [optional]  # noqa: E501
            form_fields_per_document ([SubFormFieldsPerDocumentBase]): The fields that should appear on the document, expressed as an array of objects. (We're currently fixing a bug where this property only accepts a two-dimensional array. You can read about it here: [Using Form Fields per Document](/docs/openapi/form-fields-per-document).)  **NOTE**: Fields like **text**, **dropdown**, **checkbox**, **radio**, and **hyperlink** have additional required and optional parameters. Check out the list of [additional parameters](/api/reference/constants/#form-fields-per-document) for these field types.  * Text Field use `SubFormFieldsPerDocumentText` * Dropdown Field use `SubFormFieldsPerDocumentDropdown` * Hyperlink Field use `SubFormFieldsPerDocumentHyperlink` * Checkbox Field use `SubFormFieldsPerDocumentCheckbox` * Radio Field use `SubFormFieldsPerDocumentRadio` * Signature Field use `SubFormFieldsPerDocumentSignature` * Date Signed Field use `SubFormFieldsPerDocumentDateSigned` * Initials Field use `SubFormFieldsPerDocumentInitials` * Text Merge Field use `SubFormFieldsPerDocumentTextMerge` * Checkbox Merge Field use `SubFormFieldsPerDocumentCheckboxMerge`. [optional]  # noqa: E501
            hide_text_tags (bool): Send with a value of `true` if you wish to enable automatic Text Tag removal. Defaults to `false`. When using Text Tags it is preferred that you set this to `false` and hide your tags with white text or something similar because the automatic removal system can cause unwanted clipping. See the [Text Tags](https://app.hellosign.com/api/textTagsWalkthrough#TextTagIntro) walkthrough for more details.. [optional] if omitted the server will use the default value of False  # noqa: E501
            hold_request (bool): The request from this draft will not automatically send to signers post-claim if set to `true`. Requester must [release](/api/reference/operation/signatureRequestReleaseHold/) the request from hold when ready to send. Defaults to `false`.. [optional] if omitted the server will use the default value of False  # noqa: E501
            is_for_embedded_signing (bool): The request created from this draft will also be signable in embedded mode if set to `true`. Defaults to `false`.. [optional] if omitted the server will use the default value of False  # noqa: E501
            message (str): The custom message in the email that will be sent to the signers.. [optional]  # noqa: E501
            metadata ({str: (bool, date, datetime, dict, float, int, list, str, none_type)}): Key-value data that should be attached to the signature request. This metadata is included in all API responses and events involving the signature request. For example, use the metadata field to store a signer's order number for look up when receiving events for the signature request.  Each request can include up to 10 metadata keys (or 50 nested metadata keys), with key names up to 40 characters long and values up to 1000 characters long.. [optional]  # noqa: E501
            requesting_redirect_url (str): The URL you want signers redirected to after they successfully request a signature.. [optional]  # noqa: E501
            show_preview (bool): This allows the requester to enable the editor/preview experience.  - `show_preview=true`: Allows requesters to enable the editor/preview experience. - `show_preview=false`: Allows requesters to disable the editor/preview experience.. [optional]  # noqa: E501
            show_progress_stepper (bool): When only one step remains in the signature request process and this parameter is set to `false` then the progress stepper will be hidden.. [optional] if omitted the server will use the default value of True  # noqa: E501
            signers ([SubUnclaimedDraftSigner]): Add Signers to your Unclaimed Draft Signature Request.. [optional]  # noqa: E501
            signing_options (SubSigningOptions): [optional]  # noqa: E501
            signing_redirect_url (str): The URL you want signers redirected to after they successfully sign.. [optional]  # noqa: E501
            skip_me_now (bool): Disables the \"Me (Now)\" option for the person preparing the document. Does not work with type `send_document`. Defaults to `false`.. [optional] if omitted the server will use the default value of False  # noqa: E501
            subject (str): The subject in the email that will be sent to the signers.. [optional]  # noqa: E501
            test_mode (bool): Whether this is a test, the signature request created from this draft will not be legally binding if set to `true`. Defaults to `false`.. [optional] if omitted the server will use the default value of False  # noqa: E501
            type (str): The type of the draft. By default this is `request_signature`, but you can set it to `send_document` if you want to self sign a document and download it.. [optional] if omitted the server will use the default value of "request_signature"  # noqa: E501
            use_preexisting_fields (bool): Set `use_text_tags` to `true` to enable [Text Tags](https://app.hellosign.com/api/textTagsWalkthrough#TextTagIntro) parsing in your document (defaults to disabled, or `false`). Alternatively, if your PDF contains pre-defined fields, enable the detection of these fields by setting the `use_preexisting_fields` to `true` (defaults to disabled, or `false`). Currently we only support use of either `use_text_tags` or `use_preexisting_fields` parameter, not both.. [optional] if omitted the server will use the default value of False  # noqa: E501
            use_text_tags (bool): Set `use_text_tags` to `true` to enable [Text Tags](https://app.hellosign.com/api/textTagsWalkthrough#TextTagIntro) parsing in your document (defaults to disabled, or `false`). Alternatively, if your PDF contains pre-defined fields, enable the detection of these fields by setting the `use_preexisting_fields` to `true` (defaults to disabled, or `false`). Currently we only support use of either `use_text_tags` or `use_preexisting_fields` parameter, not both.. [optional] if omitted the server will use the default value of False  # noqa: E501
            populate_auto_fill_fields (bool): Controls whether [auto fill fields](https://faq.hellosign.com/hc/en-us/articles/360051467511-Auto-Fill-Fields) can automatically populate a signer's information during signing.  ⚠️ **Note** ⚠️: Keep your signer's information safe by ensuring that the _signer on your signature request is the intended party_ before using this feature.. [optional] if omitted the server will use the default value of False  # noqa: E501
            expires_at (int, none_type): When the signature request will expire. Unsigned signatures will be moved to the expired status, and no longer signable. See [Signature Request Expiration Date](https://developers.hellosign.com/docs/signature-request/expiration/) for details.  **Note**: This does not correspond to the **expires_at** returned in the response.. [optional]  # noqa: E501
        """

        _check_type = kwargs.pop('_check_type', True)
        _spec_property_naming = kwargs.pop('_spec_property_naming', False)
        _path_to_item = kwargs.pop('_path_to_item', ())
        _configuration = kwargs.pop('_configuration', None)
        _visited_composed_classes = kwargs.pop('_visited_composed_classes', ())

        self = super(OpenApiModel, cls).__new__(cls)

        if args:
            raise ApiTypeError(
                "Invalid positional arguments=%s passed to %s. Remove those invalid positional arguments." % (
                    args,
                    self.__class__.__name__,
                ),
                path_to_item=_path_to_item,
                valid_classes=(self.__class__,),
            )

        self._data_store = {}
        self._check_type = _check_type
        self._spec_property_naming = _spec_property_naming
        self._path_to_item = _path_to_item
        self._configuration = _configuration
        self._visited_composed_classes = _visited_composed_classes + (self.__class__,)

        self.client_id = client_id
        self.requester_email_address = requester_email_address
        for var_name, var_value in kwargs.items():
            if var_name not in self.attribute_map and \
                        self._configuration is not None and \
                        self._configuration.discard_unknown_keys and \
                        self.additional_properties_type is None:
                # discard variable.
                continue
            setattr(self, var_name, var_value)
        return self

    required_properties = set([
        '_data_store',
        '_check_type',
        '_spec_property_naming',
        '_path_to_item',
        '_configuration',
        '_visited_composed_classes',
    ])

    @convert_js_args_to_python_args
    def __init__(self, client_id, requester_email_address, *args, **kwargs):  # noqa: E501
        """UnclaimedDraftCreateEmbeddedRequest - a model defined in OpenAPI

        Args:
            client_id (str): Client id of the app used to create the draft. Used to apply the branding and callback url defined for the app.
            requester_email_address (str): The email address of the user that should be designated as the requester of this draft, if the draft type is `request_signature`.

        Keyword Args:
            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _spec_property_naming (bool): True if the variable names in the input data
                                are serialized names, as specified in the OpenAPI document.
                                False if the variable names in the input data
                                are pythonic names, e.g. snake case (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            _visited_composed_classes (tuple): This stores a tuple of
                                classes that we have traveled through so that
                                if we see that class again we will not use its
                                discriminator again.
                                When traveling through a discriminator, the
                                composed schema that is
                                is traveled through is added to this set.
                                For example if Animal has a discriminator
                                petType and we pass in "Dog", and the class Dog
                                includes Animal, we move through Animal
                                once using the discriminator, and pick Dog.
                                Then in Dog, we will make an instance of the
                                Animal class but this time we won't travel
                                through its discriminator because we passed in
                                _visited_composed_classes = (Animal,)
            files ([file_type]): Use `files[]` to indicate the uploaded file(s) to send for signature.  This endpoint requires either **files** or **file_urls[]**, but not both.. [optional]  # noqa: E501
            file_urls ([str]): Use `file_urls[]` to have Dropbox Sign download the file(s) to send for signature.  This endpoint requires either **files** or **file_urls[]**, but not both.. [optional]  # noqa: E501
            allow_ccs (bool): This allows the requester to specify whether the user is allowed to provide email addresses to CC when claiming the draft.. [optional] if omitted the server will use the default value of True  # noqa: E501
            allow_decline (bool): Allows signers to decline to sign a document if `true`. Defaults to `false`.. [optional] if omitted the server will use the default value of False  # noqa: E501
            allow_reassign (bool): Allows signers to reassign their signature requests to other signers if set to `true`. Defaults to `false`.  **Note**: Only available for Premium plan and higher.. [optional] if omitted the server will use the default value of False  # noqa: E501
            attachments ([SubAttachment]): A list describing the attachments. [optional]  # noqa: E501
            cc_email_addresses ([str]): The email addresses that should be CCed.. [optional]  # noqa: E501
            custom_fields ([SubCustomField]): When used together with merge fields, `custom_fields` allows users to add pre-filled data to their signature requests.  Pre-filled data can be used with \"send-once\" signature requests by adding merge fields with `form_fields_per_document` or [Text Tags](https://app.hellosign.com/api/textTagsWalkthrough#TextTagIntro) while passing values back with `custom_fields` together in one API call.  For using pre-filled on repeatable signature requests, merge fields are added to templates in the Dropbox Sign UI or by calling [/template/create_embedded_draft](/api/reference/operation/templateCreateEmbeddedDraft) and then passing `custom_fields` on subsequent signature requests referencing that template.. [optional]  # noqa: E501
            editor_options (SubEditorOptions): [optional]  # noqa: E501
            field_options (SubFieldOptions): [optional]  # noqa: E501
            force_signer_page (bool): Provide users the ability to review/edit the signers.. [optional] if omitted the server will use the default value of False  # noqa: E501
            force_subject_message (bool): Provide users the ability to review/edit the subject and message.. [optional] if omitted the server will use the default value of False  # noqa: E501
            form_field_groups ([SubFormFieldGroup]): Group information for fields defined in `form_fields_per_document`. String-indexed JSON array with `group_label` and `requirement` keys. `form_fields_per_document` must contain fields referencing a group defined in `form_field_groups`.. [optional]  # noqa: E501
            form_field_rules ([SubFormFieldRule]): Conditional Logic rules for fields defined in `form_fields_per_document`.. [optional]  # noqa: E501
            form_fields_per_document ([SubFormFieldsPerDocumentBase]): The fields that should appear on the document, expressed as an array of objects. (We're currently fixing a bug where this property only accepts a two-dimensional array. You can read about it here: [Using Form Fields per Document](/docs/openapi/form-fields-per-document).)  **NOTE**: Fields like **text**, **dropdown**, **checkbox**, **radio**, and **hyperlink** have additional required and optional parameters. Check out the list of [additional parameters](/api/reference/constants/#form-fields-per-document) for these field types.  * Text Field use `SubFormFieldsPerDocumentText` * Dropdown Field use `SubFormFieldsPerDocumentDropdown` * Hyperlink Field use `SubFormFieldsPerDocumentHyperlink` * Checkbox Field use `SubFormFieldsPerDocumentCheckbox` * Radio Field use `SubFormFieldsPerDocumentRadio` * Signature Field use `SubFormFieldsPerDocumentSignature` * Date Signed Field use `SubFormFieldsPerDocumentDateSigned` * Initials Field use `SubFormFieldsPerDocumentInitials` * Text Merge Field use `SubFormFieldsPerDocumentTextMerge` * Checkbox Merge Field use `SubFormFieldsPerDocumentCheckboxMerge`. [optional]  # noqa: E501
            hide_text_tags (bool): Send with a value of `true` if you wish to enable automatic Text Tag removal. Defaults to `false`. When using Text Tags it is preferred that you set this to `false` and hide your tags with white text or something similar because the automatic removal system can cause unwanted clipping. See the [Text Tags](https://app.hellosign.com/api/textTagsWalkthrough#TextTagIntro) walkthrough for more details.. [optional] if omitted the server will use the default value of False  # noqa: E501
            hold_request (bool): The request from this draft will not automatically send to signers post-claim if set to `true`. Requester must [release](/api/reference/operation/signatureRequestReleaseHold/) the request from hold when ready to send. Defaults to `false`.. [optional] if omitted the server will use the default value of False  # noqa: E501
            is_for_embedded_signing (bool): The request created from this draft will also be signable in embedded mode if set to `true`. Defaults to `false`.. [optional] if omitted the server will use the default value of False  # noqa: E501
            message (str): The custom message in the email that will be sent to the signers.. [optional]  # noqa: E501
            metadata ({str: (bool, date, datetime, dict, float, int, list, str, none_type)}): Key-value data that should be attached to the signature request. This metadata is included in all API responses and events involving the signature request. For example, use the metadata field to store a signer's order number for look up when receiving events for the signature request.  Each request can include up to 10 metadata keys (or 50 nested metadata keys), with key names up to 40 characters long and values up to 1000 characters long.. [optional]  # noqa: E501
            requesting_redirect_url (str): The URL you want signers redirected to after they successfully request a signature.. [optional]  # noqa: E501
            show_preview (bool): This allows the requester to enable the editor/preview experience.  - `show_preview=true`: Allows requesters to enable the editor/preview experience. - `show_preview=false`: Allows requesters to disable the editor/preview experience.. [optional]  # noqa: E501
            show_progress_stepper (bool): When only one step remains in the signature request process and this parameter is set to `false` then the progress stepper will be hidden.. [optional] if omitted the server will use the default value of True  # noqa: E501
            signers ([SubUnclaimedDraftSigner]): Add Signers to your Unclaimed Draft Signature Request.. [optional]  # noqa: E501
            signing_options (SubSigningOptions): [optional]  # noqa: E501
            signing_redirect_url (str): The URL you want signers redirected to after they successfully sign.. [optional]  # noqa: E501
            skip_me_now (bool): Disables the \"Me (Now)\" option for the person preparing the document. Does not work with type `send_document`. Defaults to `false`.. [optional] if omitted the server will use the default value of False  # noqa: E501
            subject (str): The subject in the email that will be sent to the signers.. [optional]  # noqa: E501
            test_mode (bool): Whether this is a test, the signature request created from this draft will not be legally binding if set to `true`. Defaults to `false`.. [optional] if omitted the server will use the default value of False  # noqa: E501
            type (str): The type of the draft. By default this is `request_signature`, but you can set it to `send_document` if you want to self sign a document and download it.. [optional] if omitted the server will use the default value of "request_signature"  # noqa: E501
            use_preexisting_fields (bool): Set `use_text_tags` to `true` to enable [Text Tags](https://app.hellosign.com/api/textTagsWalkthrough#TextTagIntro) parsing in your document (defaults to disabled, or `false`). Alternatively, if your PDF contains pre-defined fields, enable the detection of these fields by setting the `use_preexisting_fields` to `true` (defaults to disabled, or `false`). Currently we only support use of either `use_text_tags` or `use_preexisting_fields` parameter, not both.. [optional] if omitted the server will use the default value of False  # noqa: E501
            use_text_tags (bool): Set `use_text_tags` to `true` to enable [Text Tags](https://app.hellosign.com/api/textTagsWalkthrough#TextTagIntro) parsing in your document (defaults to disabled, or `false`). Alternatively, if your PDF contains pre-defined fields, enable the detection of these fields by setting the `use_preexisting_fields` to `true` (defaults to disabled, or `false`). Currently we only support use of either `use_text_tags` or `use_preexisting_fields` parameter, not both.. [optional] if omitted the server will use the default value of False  # noqa: E501
            populate_auto_fill_fields (bool): Controls whether [auto fill fields](https://faq.hellosign.com/hc/en-us/articles/360051467511-Auto-Fill-Fields) can automatically populate a signer's information during signing.  ⚠️ **Note** ⚠️: Keep your signer's information safe by ensuring that the _signer on your signature request is the intended party_ before using this feature.. [optional] if omitted the server will use the default value of False  # noqa: E501
            expires_at (int, none_type): When the signature request will expire. Unsigned signatures will be moved to the expired status, and no longer signable. See [Signature Request Expiration Date](https://developers.hellosign.com/docs/signature-request/expiration/) for details.  **Note**: This does not correspond to the **expires_at** returned in the response.. [optional]  # noqa: E501
        """

        _check_type = kwargs.pop('_check_type', True)
        _spec_property_naming = kwargs.pop('_spec_property_naming', False)
        _path_to_item = kwargs.pop('_path_to_item', ())
        _configuration = kwargs.pop('_configuration', None)
        _visited_composed_classes = kwargs.pop('_visited_composed_classes', ())

        if args:
            raise ApiTypeError(
                "Invalid positional arguments=%s passed to %s. Remove those invalid positional arguments." % (
                    args,
                    self.__class__.__name__,
                ),
                path_to_item=_path_to_item,
                valid_classes=(self.__class__,),
            )

        self._data_store = {}
        self._check_type = _check_type
        self._spec_property_naming = _spec_property_naming
        self._path_to_item = _path_to_item
        self._configuration = _configuration
        self._visited_composed_classes = _visited_composed_classes + (self.__class__,)

        self.client_id = client_id
        self.requester_email_address = requester_email_address
        for var_name, var_value in kwargs.items():
            if var_name not in self.attribute_map and \
                        self._configuration is not None and \
                        self._configuration.discard_unknown_keys and \
                        self.additional_properties_type is None:
                # discard variable.
                continue
            setattr(self, var_name, var_value)
            if var_name in self.read_only_vars:
                raise ApiAttributeError(f"`{var_name}` is a read-only attribute. Use `from_openapi_data` to instantiate "
                                     f"class with read only attributes.")



import unittest

from dropbox_sign import ApiClient, Configuration, apis, models as m
from test_utils import get_fixture_data, MockPoolManager, get_base_path


class TestSignatureRequestApi(unittest.TestCase):
    def setUp(self):
        self.configuration = Configuration()
        self.api_client = ApiClient(self.configuration)
        self.mock_pool = MockPoolManager(self)
        self.api_client.rest_client.pool_manager = self.mock_pool

        self.api = apis.SignatureRequestApi(self.api_client)

    def test_init_allows_binary_file(self):
        data = {
            "test_mode": True,
            "title": "NDA with Acme Co.",
            "subject": "The NDA we talked about",
            "message": "Please sign this NDA and then we can discuss more.",
            "signers": [
                {
                    "email_address": "jill@example.com",
                    "name": "Jill",
                    "order": 1
                }
            ],
            "form_fields_per_document": [
                {
                    "type": "signature",
                    "document_index": 0,
                    "api_id": "4688957689",
                    "name": "signature1",
                    "x": 5,
                    "y": 7,
                    "width": 60,
                    "height": 30,
                    "required": True,
                    "signer": "0",
                    "page": 1,
                    "placeholder": "My placeholder value",
                }
            ],
            "files": [open(get_base_path() + "/../test_fixtures/pdf-sample.pdf", "rb")]
        }

        obj = m.SignatureRequestSendRequest.init(data)

        self.assertEqual(data["test_mode"], obj.test_mode)
        self.assertEqual(data["title"], obj.title)
        self.assertEqual(data["subject"], obj.subject)
        self.assertEqual(data["message"], obj.message)

        self.assertEqual(data["signers"][0]["email_address"], obj.signers[0].email_address)
        self.assertEqual(data["signers"][0]["name"], obj.signers[0].name)
        self.assertEqual(data["signers"][0]["order"], obj.signers[0].order)

        self.assertEqual(data["form_fields_per_document"][0]["type"], obj.form_fields_per_document[0].type)
        self.assertEqual(data["form_fields_per_document"][0]["document_index"], obj.form_fields_per_document[0].document_index)
        self.assertEqual(data["form_fields_per_document"][0]["api_id"], obj.form_fields_per_document[0].api_id)
        self.assertEqual(data["form_fields_per_document"][0]["name"], obj.form_fields_per_document[0].name)
        self.assertEqual(data["form_fields_per_document"][0]["x"], obj.form_fields_per_document[0].x)
        self.assertEqual(data["form_fields_per_document"][0]["y"], obj.form_fields_per_document[0].y)
        self.assertEqual(data["form_fields_per_document"][0]["width"], obj.form_fields_per_document[0].width)
        self.assertEqual(data["form_fields_per_document"][0]["height"], obj.form_fields_per_document[0].height)
        self.assertEqual(data["form_fields_per_document"][0]["required"], obj.form_fields_per_document[0].required)
        self.assertEqual(data["form_fields_per_document"][0]["signer"], obj.form_fields_per_document[0].signer)
        self.assertEqual(data["form_fields_per_document"][0]["page"], obj.form_fields_per_document[0].page)
        self.assertEqual(data["form_fields_per_document"][0]["placeholder"], obj.form_fields_per_document[0].placeholder)

        self.assertEqual(data["files"][0], obj.files[0])

        obj.files[0].close()

    def test_signature_request_bulk_create_embedded_with_template(self):
        request_class = 'SignatureRequestBulkCreateEmbeddedWithTemplateRequest'
        request_data = get_fixture_data(request_class)['default']

        response_class = 'BulkSendJobSendResponse'
        response_data = get_fixture_data(response_class)['default']

        self.mock_pool.expect_request(
            content_type='multipart/form-data',
            data=request_data,
            response=response_data
        )
        expected = m.BulkSendJobSendResponse.init(response_data)
        obj = m.SignatureRequestBulkCreateEmbeddedWithTemplateRequest.init(request_data)
        obj.signer_file = open(f'{get_base_path()}/bulk-send-sample.csv', 'rb')

        result = self.api.signature_request_bulk_create_embedded_with_template(obj)

        self.assertEqual(result.__class__.__name__, response_class)
        self.assertEqual(result, expected)

        obj.signer_file.close()

    def test_signature_request_bulk_send_with_template(self):
        request_class = 'SignatureRequestBulkSendWithTemplateRequest'
        request_data = get_fixture_data(request_class)['default']

        response_class = 'BulkSendJobSendResponse'
        response_data = get_fixture_data(response_class)['default']

        self.mock_pool.expect_request(
            content_type='multipart/form-data',
            data=request_data,
            response=response_data
        )
        expected = m.BulkSendJobSendResponse.init(response_data)
        obj = m.SignatureRequestBulkSendWithTemplateRequest.init(request_data)
        obj.signer_file = open(f'{get_base_path()}/bulk-send-sample.csv', 'rb')

        result = self.api.signature_request_bulk_send_with_template(obj)

        self.assertEqual(result.__class__.__name__, response_class)
        self.assertEqual(result, expected)

        obj.signer_file.close()

    def test_signature_request_create_embedded(self):
        request_class = 'SignatureRequestCreateEmbeddedRequest'
        request_data = get_fixture_data(request_class)['default']

        response_class = 'SignatureRequestGetResponse'
        response_data = get_fixture_data(response_class)['default']

        self.mock_pool.expect_request(
            content_type='multipart/form-data',
            data=request_data,
            response=response_data
        )
        expected = m.SignatureRequestGetResponse.init(response_data)
        obj = m.SignatureRequestCreateEmbeddedRequest.init(request_data)
        obj.files = [open(f'{get_base_path()}/pdf-sample.pdf', 'rb')]

        result = self.api.signature_request_create_embedded(obj)

        self.assertEqual(result.__class__.__name__, response_class)
        self.assertEqual(result, expected)

        obj.files[0].close()

    def test_signature_request_create_embedded_with_template(self):
        request_class = 'SignatureRequestCreateEmbeddedWithTemplateRequest'
        request_data = get_fixture_data(request_class)['default']

        response_class = 'SignatureRequestGetResponse'
        response_data = get_fixture_data(response_class)['default']

        self.mock_pool.expect_request(
            content_type='multipart/form-data',
            data=request_data,
            response=response_data
        )
        expected = m.SignatureRequestGetResponse.init(response_data)
        obj = m.SignatureRequestCreateEmbeddedWithTemplateRequest.init(request_data)
        obj.files = [open(f'{get_base_path()}/pdf-sample.pdf', 'rb')]

        result = self.api.signature_request_create_embedded_with_template(obj)

        self.assertEqual(result.__class__.__name__, response_class)
        self.assertEqual(result, expected)

        obj.files[0].close()

    def test_signature_request_files(self):
        self.skipTest('skipping test_signature_request_files')

    def test_signature_request_get(self):
        signature_request_id = 'fa5c8a0b0f492d768749333ad6fcc214c111e967'

        response_class = 'SignatureRequestGetResponse'
        response_data = get_fixture_data(response_class)['default']

        self.mock_pool.expect_request(
            content_type='application/json',
            response=response_data
        )
        expected = m.SignatureRequestGetResponse.init(response_data)

        result = self.api.signature_request_get(signature_request_id)

        self.assertEqual(result.__class__.__name__, response_class)
        self.assertEqual(result, expected)

    def test_signature_request_list(self):
        account_id = 'all'

        response_class = 'SignatureRequestListResponse'
        response_data = get_fixture_data(response_class)['default']

        self.mock_pool.expect_request(
            content_type='application/json',
            response=response_data
        )
        expected = m.SignatureRequestListResponse.init(response_data)

        result = self.api.signature_request_list(account_id=account_id)

        self.assertEqual(result.__class__.__name__, response_class)
        self.assertEqual(result, expected)

    def test_signature_request_list_null_query_value_removed(self):
        account_id = None

        self.mock_pool.expect_request(
            content_type='application/json',
            response=None,
        )

        self.api.signature_request_list(account_id=account_id)

        request_fields = self.mock_pool.get_fields()
        self.assertTrue(not request_fields)

        account_id = None
        query = None

        self.mock_pool.expect_request(
            content_type='application/json',
            response=None,
        )

        self.api.signature_request_list(account_id=account_id, query=query)

        request_fields = self.mock_pool.get_fields()
        self.assertTrue(not request_fields)

        account_id = 'ABC123'
        query = None

        self.mock_pool.expect_request(
            content_type='application/json',
            response=None,
        )

        self.api.signature_request_list(account_id=account_id, query=query)

        request_fields = self.mock_pool.get_fields()
        expected_fields = [
            ('account_id', account_id),
        ]
        self.assertTrue(expected_fields == request_fields)

        account_id = 'ABC123'
        query = 'My amazing query'

        self.mock_pool.expect_request(
            content_type='application/json',
            response=None,
        )

        self.api.signature_request_list(account_id=account_id, query=query)

        request_fields = self.mock_pool.get_fields()
        expected_fields = [
            ('account_id', account_id),
            ('query', query),
        ]
        self.assertTrue(expected_fields == request_fields)

    def test_signature_request_release_hold(self):
        signature_request_id = 'fa5c8a0b0f492d768749333ad6fcc214c111e967'

        response_class = 'SignatureRequestGetResponse'
        response_data = get_fixture_data(response_class)['default']

        self.mock_pool.expect_request(
            content_type='application/json',
            response=response_data
        )
        expected = m.SignatureRequestGetResponse.init(response_data)

        result = self.api.signature_request_release_hold(signature_request_id)

        self.assertEqual(result.__class__.__name__, response_class)
        self.assertEqual(result, expected)

    def test_signature_request_remind(self):
        signature_request_id = 'fa5c8a0b0f492d768749333ad6fcc214c111e967'

        request_class = 'SignatureRequestRemindRequest'
        request_data = get_fixture_data(request_class)['default']

        response_class = 'SignatureRequestGetResponse'
        response_data = get_fixture_data(response_class)['default']

        self.mock_pool.expect_request(
            content_type='application/json',
            data=request_data,
            response=response_data
        )
        expected = m.SignatureRequestGetResponse.init(response_data)
        obj = m.SignatureRequestRemindRequest.init(request_data)

        result = self.api.signature_request_remind(signature_request_id, obj)

        self.assertEqual(result.__class__.__name__, response_class)
        self.assertEqual(result, expected)

    def test_signature_request_remove(self):
        self.skipTest('skipping test_signature_request_remove')

    def test_signature_request_send_request(self):
        request_class = 'SignatureRequestSendRequest'
        request_data = get_fixture_data(request_class)['default']

        response_class = 'SignatureRequestGetResponse'
        response_data = get_fixture_data(response_class)['default']

        self.mock_pool.expect_request(
            content_type='multipart/form-data',
            data=request_data,
            response=response_data
        )
        expected = m.SignatureRequestGetResponse.init(response_data)
        obj = m.SignatureRequestSendRequest.init(request_data)
        obj.files = [open(f'{get_base_path()}/pdf-sample.pdf', 'rb')]

        result = self.api.signature_request_send(obj)

        self.assertEqual(result.__class__.__name__, response_class)
        self.assertEqual(result, expected)

        obj.files[0].close()

    def test_no_file_forces_application_json(self):
        request_class = 'SignatureRequestSendRequest'
        request_data = get_fixture_data(request_class)['default']

        response_class = 'SignatureRequestGetResponse'
        response_data = get_fixture_data(response_class)['default']

        self.mock_pool.expect_request(
            content_type='application/json',
            data=request_data,
            response=response_data
        )
        expected = m.SignatureRequestGetResponse.init(response_data)
        obj = m.SignatureRequestSendRequest.init(request_data)

        result = self.api.signature_request_send(obj)

        self.assertEqual(result.__class__.__name__, response_class)
        self.assertEqual(result, expected)

    def test_signature_request_send_with_template(self):
        request_class = 'SignatureRequestSendWithTemplateRequest'
        request_data = get_fixture_data(request_class)['default']

        response_class = 'SignatureRequestGetResponse'
        response_data = get_fixture_data(response_class)['default']

        self.mock_pool.expect_request(
            content_type='application/json',
            data=request_data,
            response=response_data
        )
        expected = m.SignatureRequestGetResponse.init(response_data)
        obj = m.SignatureRequestSendWithTemplateRequest.init(request_data)

        result = self.api.signature_request_send_with_template(obj)

        self.assertEqual(result.__class__.__name__, response_class)
        self.assertEqual(result, expected)

    def test_signature_request_update(self):
        signature_request_id = 'fa5c8a0b0f492d768749333ad6fcc214c111e967'

        request_class = 'SignatureRequestUpdateRequest'
        request_data = get_fixture_data(request_class)['default']

        response_class = 'SignatureRequestGetResponse'
        response_data = get_fixture_data(response_class)['default']

        self.mock_pool.expect_request(
            content_type='application/json',
            data=request_data,
            response=response_data
        )
        expected = m.SignatureRequestGetResponse.init(response_data)
        obj = m.SignatureRequestUpdateRequest.init(request_data)

        result = self.api.signature_request_update(signature_request_id, obj)

        self.assertEqual(result.__class__.__name__, response_class)
        self.assertEqual(result, expected)


if __name__ == '__main__':
    unittest.main()



import unittest

from dropbox_sign import models
from dropbox_sign.models import SignatureRequestSendRequest
from test_utils import get_fixture_data


class TestSubFormFieldsPerDocumentBase(unittest.TestCase):
    def test_is_valid(self):
        fixture_data = get_fixture_data('SubFormFieldsPerDocument')

        for ftype, data in fixture_data.items():
            payload = {
                'signers': [],
                'form_fields_per_document': [data]
            }

            obj = SignatureRequestSendRequest.init(payload)

            form_fields_per_document = obj.form_fields_per_document[0]
            class_type = eval(f'models.{ftype}')

            self.assertEqual(form_fields_per_document.__class__.__name__, class_type.__name__)
            self.assertEqual(form_fields_per_document.to_dict(), data)

    def test_signer_allows_int(self):
        fixture_data = get_fixture_data('SubFormFieldsPerDocument')

        for ftype, data in fixture_data.items():
            data['signer'] = 1234

            payload = {
                'signers': [],
                'form_fields_per_document': [data]
            }

            expected_signer = 1234

            obj = SignatureRequestSendRequest.init(payload)
            form_fields_per_document = obj.form_fields_per_document[0]
            self.assertEqual(form_fields_per_document.signer, expected_signer)

    def test_signer_allows_string(self):
        fixture_data = get_fixture_data('SubFormFieldsPerDocument')

        for ftype, data in fixture_data.items():
            data['signer'] = 'sender'

            payload = {
                'signers': [],
                'form_fields_per_document': [data]
            }

            expected_signer = 'sender'

            obj = SignatureRequestSendRequest.init(payload)
            form_fields_per_document = obj.form_fields_per_document[0]
            self.assertEqual(form_fields_per_document.signer, expected_signer)


if __name__ == '__main__':
    unittest.main()
